@@ -1,5 +1,5 @@
 // ResourceTreeBrowser.java
-// $Id: ResourceTreeBrowser.java,v 1.30.4.1 1999/11/24 16:46:53 bmahe Exp $
+// $Id: ResourceTreeBrowser.java,v 1.30.4.2 2000/02/18 15:14:05 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1998.
 // Please first read the full copyright statement in file COPYRIGHT.html
  
@@ -59,7 +59,7 @@ import org.w3c.jigsaw.admin.RemoteResource;
 
 /**
  * A JTree used to manage RemoteResource.
- * @version $Revision: 1.30.4.1 $
+ * @version $Revision: 1.30.4.2 $
  * @author  Benot Mah (bmahe@w3.org)
  */
 public class ResourceTreeBrowser extends JTree 
@@ -264,7 +264,7 @@ public class ResourceTreeBrowser extends JTree
 	    if (cmd == ResourceActionEvent.DELETE_EVENT) {
 		deleteSelectedResources();
 	    } else if (cmd == ResourceActionEvent.REINDEX_EVENT) {
-		reindexSelectedResources();
+		reindexSelectedResources(false); //FIXME
 	    } else if (cmd == ResourceActionEvent.REFERENCE_EVENT) {
 		showReferenceDocumentation();
 	    } else if (cmd == ResourceActionEvent.ADD_EVENT) {
@@ -508,8 +508,9 @@ public class ResourceTreeBrowser extends JTree
      * Reindex the containers associated to the selected nodes.
      * Display an error message (dialog) if there is no node selected or
      * if one of the selected resource is not a ResourceContainer.
+     * @param rec recursivly?
      */
-    protected void reindexSelectedResources() {
+    protected void reindexSelectedResources(boolean rec) {
 	TreePath path[] = removeDescendants(getSelectionPaths());
 	if (path == null) {
 	    JOptionPane.showMessageDialog(this, 
@@ -531,7 +532,7 @@ public class ResourceTreeBrowser extends JTree
 		    if (rrw == null)
 			continue;
 		    try {
-			reindexResource(rrw);
+			reindexResource(rrw, rec);
 		    } catch (RemoteAccessException ex) {
 			Message.showErrorMessage(this, ex);
 			continue;
@@ -546,14 +547,15 @@ public class ResourceTreeBrowser extends JTree
      * Display an error message (dialog) if the resource
      * is not a ResourceContainer.
      * @param rrw the RemoteResourceWrapper
+     * @param rec recursivly?
      * @exception RemoteAccessException if a Remote Error occurs
      */
-    protected void reindexResource(RemoteResourceWrapper rrw) 
+    protected void reindexResource(RemoteResourceWrapper rrw, boolean rec) 
 	throws RemoteAccessException
     {
 	RemoteResource rr = rrw.getResource();
 	if (rr.isContainer()) {
-	    rr.reindex();
+	    rr.reindex(rec);
 	} else {
 	    JOptionPane.
 		showMessageDialog(this, 
@@ -712,7 +714,9 @@ public class ResourceTreeBrowser extends JTree
 	    } else if (command.equals("add")) {
 		addResourceToSelectedContainer();
 	    } else if (command.equals("reindex")) {
-		reindexSelectedResources();
+		reindexSelectedResources(true);
+	    } else if (command.equals("reindex-locally")) {
+		reindexSelectedResources(false);
 	    } else if (command.equals("info")) {
 		showReferenceDocumentation();
 	    } else if (command.equals("edit")) {
@@ -749,6 +753,11 @@ public class ResourceTreeBrowser extends JTree
 	    menuItem.addActionListener(pmal);
 	    menuItem.setActionCommand("reindex");
 	    popupMenu.add(menuItem);
+
+	    menuItem = new JMenuItem("Reindex Locally", Icons.reindexIcon);
+	    menuItem.addActionListener(pmal);
+	    menuItem.setActionCommand("reindex-locally");
+	    popupMenu.add(menuItem);
 	    
 	    menuItem = new JMenuItem("Add resource", Icons.addIcon);
 	    menuItem.addActionListener(pmal);
