@@ -1,5 +1,5 @@
 // ResourceActionSource.java
-// $Id: ResourceActionSource.java,v 1.3 1999/03/05 13:58:20 bmahe Exp $
+// $Id: ResourceActionSource.java,v 1.3.4.1 1999/10/12 12:04:36 ylafon Exp $
 // (c) COPYRIGHT MIT, INRIA and Keio, 1999.
 // Please first read the full copyright statement in file COPYRIGHT.html
  
@@ -7,10 +7,10 @@ package org.w3c.jigadmin.events;
 
 /**
  * Object that implements this interface cand fire ResourceActionEvent.
- * @version $Revision: 1.3 $
+ * @version $Revision: 1.3.4.1 $
  * @author  Benot Mah (bmahe@w3.org)
  */
-abstract public interface ResourceActionSource {
+public interface ResourceActionSource {
 
     /**
      * Add a ResourceActionListener.
