@@ -1,5 +1,5 @@
 // CvsEntryResource.java
-// $Id: CvsEntryResource.java,v 1.16 1998/10/30 11:02:36 bmahe Exp $
+// $Id: CvsEntryResource.java,v 1.16.4.1 1999/10/12 11:50:21 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -97,7 +97,7 @@ public class CvsEntryResource extends ContainerResource {
 			ContainerResource dir = (ContainerResource) parent;
 			ResourceReference rr_res = dir.lookup(name);
 			if (rr_res == null) {
-			    setValue(ATTR_CONTENT_TYPE,
+			    this.setValue(ATTR_CONTENT_TYPE,
 				     org.w3c.www.mime.Utils.getMimeType(name));
 			    return;
 			}
@@ -108,12 +108,12 @@ public class CvsEntryResource extends ContainerResource {
 				    res.getFrame(
 					 Class.forName(
 					   "org.w3c.jigsaw.frames.HTTPFrame"));
-				setValue(ATTR_CONTENT_LANGUAGE,
-					 httpFrame.getContentLanguage());
-				setValue(ATTR_CONTENT_ENCODING,
-					 httpFrame.getContentEncoding());
-				setValue(ATTR_CONTENT_TYPE,
-					 httpFrame.getContentType());
+				this.setValue(ATTR_CONTENT_LANGUAGE,
+					      httpFrame.getContentLanguage());
+				this.setValue(ATTR_CONTENT_ENCODING,
+					      httpFrame.getContentEncoding());
+				this.setValue(ATTR_CONTENT_TYPE,
+					      httpFrame.getContentType());
 			    }
 			} catch (InvalidResourceException ex) {
 			    //nothing to do ;(
@@ -162,7 +162,8 @@ public class CvsEntryResource extends ContainerResource {
 		    byte content[] = out.toByteArray();
 		    ByteArrayInputStream in = 
 			new ByteArrayInputStream(content);
-		    setValue(ATTR_CONTENT_LENGTH, new Integer(content.length));
+		    this.setValue(ATTR_CONTENT_LENGTH, 
+				  new Integer(content.length));
 		    Reply reply = createDefaultReply(request, HTTP.OK);
 		    reply.setStream(in);
 		    return reply;
