@@ -1,5 +1,5 @@
 // BrokerFrame.java
-// $Id: BrokerFrame.java,v 1.7.4.1 1999/12/01 10:08:38 bmahe Exp $
+// $Id: BrokerFrame.java,v 1.7.4.2 2000/02/18 15:14:05 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1997.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -594,7 +594,7 @@ public class BrokerFrame extends HTTPFrame {
 	return null;
     }
 
-    public Reply remoteReindexResource(Request request) 
+    public Reply remoteReindexResource(Request request, boolean rec) 
 	throws ProtocolException
     {
 	// Check target resource class:
@@ -606,7 +606,7 @@ public class BrokerFrame extends HTTPFrame {
 		if (r instanceof org.w3c.tools.resources.DirectoryResource) {
 		    org.w3c.tools.resources.DirectoryResource dir = 
 			(org.w3c.tools.resources.DirectoryResource) r;
-		    dir.reindex();
+		    dir.reindex(rec);
 		    return okReply(request);
 		} else {
 		    error(request, "Can't reindex this resource"+
@@ -680,7 +680,9 @@ public class BrokerFrame extends HTTPFrame {
 	} else if (mth.equals("DELETE-RESOURCE") ) {
 	    return remoteDeleteResource(request);
 	} else if (mth.equals("REINDEX-RESOURCE") ) {
-	    return remoteReindexResource(request);
+	    return remoteReindexResource(request, true);
+	} else if (mth.equals("REINDEX-LOCALLY") ) {
+	    return remoteReindexResource(request, false);
 	} else if (mth.equals("UNREGISTER-FRAME")) {
 	    checkContentType(request);
 	    return remoteUnregisterFrame(request);
