@@ -1,5 +1,5 @@
 // PlainRemoteResource.java
-// $Id: PlainRemoteResource.java,v 1.25.4.1 1999/12/01 10:08:38 bmahe Exp $
+// $Id: PlainRemoteResource.java,v 1.25.4.2 2000/02/18 15:14:05 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1997.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -106,14 +106,18 @@ public class PlainRemoteResource implements RemoteResource {
      * Reindex the resource's children if this resource is a DirectoryResource.
      * @exception RemoteAccessException If it's not a DirectoryResource
      */
-    public void reindex()
+    public void reindex(boolean rec)
 	throws RemoteAccessException
     {
 	if (isDirectoryResource()) {
 	    try {
 		Request req = createRequest();
 		// Prepare the request:
-		req.setMethod("REINDEX-RESOURCE");
+		if (rec) {
+		    req.setMethod("REINDEX-RESOURCE");
+		} else {
+		    req.setMethod("REINDEX-LOCALLY");
+		}
 		// Run it:
 		Reply rep = admin.runRequest(req);
 	    } catch (RemoteAccessException rae) {
