@@ -1,5 +1,5 @@
 // CgiFrame.java
-// $Id: CgiFrame.java,v 1.8 1999/03/26 16:26:31 ylafon Exp $
+// $Id: CgiFrame.java,v 1.9.4.4 2000/06/05 17:26:19 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -378,6 +378,21 @@ public class CgiFrame extends HTTPFrame {
 	return sb.toString();
     }
 
+    /**
+     * Get this resource Etag
+     * @return an instance of HttpEntityTag, or <strong>null</strong> if not
+     *    defined.
+     */
+    public HttpEntityTag getETag() {
+	if (etag == null) {
+	    HttpEntityTag netag = super.getETag();
+	    if (netag != null) {
+		etag.setWeak(true);
+	    }
+	}
+	return etag;
+    }
+
     /**
      * Handle the CGI script output.
      * This methods handles the CGI script output. Depending on the
@@ -433,12 +448,18 @@ public class CgiFrame extends HTTPFrame {
 		    reply.setContent("CGI script emited invalid status.");
 		    throw new HTTPException(reply);
 		}
-		reply = request.makeReply(status);
+		// we will use the default for this frame, but remove
+		// the length calculated from the script size.
+		reply = createDefaultReply(request, status);
+		reply.setContentLength(-1);
 	    } else {
 		// No status code available, any location header ?
-		reply = request.makeReply((location == null)
-					  ? HTTP.OK
-					  : HTTP.FOUND);
+		if (location != null) {
+		    reply = request.makeReply(HTTP.FOUND);
+		} else {
+		    reply = createDefaultReply(request, HTTP.OK);
+		    reply.setContentLength(-1);
+		}
 	    }
 	    // Set up the location header if needed:
 	    if ( location != null ) {
@@ -602,7 +623,7 @@ public class CgiFrame extends HTTPFrame {
 	    System.arraycopy(command, 0, run, 1, command.length);
 	    return Runtime.getRuntime().exec (run, aenv) ;
 	} else {
-	    return Runtime.getRuntime().exec (command[0], aenv) ;
+	    return Runtime.getRuntime().exec (command, aenv) ;
 	}
     }
 
