@@ -1,5 +1,5 @@
 // HTTPFrame.java
-// $Id: HTTPFrame.java,v 1.63.4.6 2000/01/20 17:46:51 ylafon Exp $
+// $Id: HTTPFrame.java,v 1.63.4.14 2000/06/23 12:29:54 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1997.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -134,6 +134,10 @@ public class HTTPFrame extends ProtocolFrame {
      * Attribute index - our index resource name.
      */
     protected static int ATTR_INDEX = -1 ;
+    /**
+     * Attribute index - our indexes resource name.
+     */
+    protected static int ATTR_INDEXES = -1 ;
     /**
      * Attribute index - The icon directory to use in dir listing.
      */
@@ -238,6 +242,11 @@ public class HTTPFrame extends ProtocolFrame {
 				, null
 				, Attribute.EDITABLE) ;
 	ATTR_INDEX = AttributeRegistry.registerAttribute(cls, a) ;
+	// Our indexes resource name 
+	a = new StringArrayAttribute("indexes"
+				     , null
+				     , Attribute.EDITABLE) ;
+	ATTR_INDEXES = AttributeRegistry.registerAttribute(cls, a) ;
 	// Our icon directory.
 	a = new StringAttribute("icondir"
 				, null
@@ -545,6 +554,10 @@ public class HTTPFrame extends ProtocolFrame {
 			+ Long.toString(lstamp, 32);
 		}
 	    }
+            // no luck, exit
+            if (etag_s == null) {
+                return null;
+            }
 	    etag = HttpFactory.makeETag(false, etag_s);
 	}
 	return etag;
@@ -764,13 +777,38 @@ public class HTTPFrame extends ProtocolFrame {
     }
 
     /**
-     * Get the optinal index name for this directory listing.
+     * Get the optional main index name for this directory listing.
      * @return The name of the resource responsible to list that container.
      */
     public String getIndex() {
 	return (String) getValue(ATTR_INDEX, null) ;
     }
 
+    /**
+     * Get the optional index name array for this directory listing.
+     * @return The index name array (including the main index)
+     * @see #getIndex
+     */
+    public String[] getIndexes() {
+	String mainIndex = getIndex();
+	if (mainIndex != null) {
+	    String indexes[] = (String[]) getValue(ATTR_INDEXES, null);
+	    if (indexes != null) {
+		int    len          = indexes.length + 1;
+		String mergeIndex[] = new String[len];
+		mergeIndex[0] = mainIndex;
+		System.arraycopy(indexes, 0, mergeIndex, 1, len-1);
+		return mergeIndex;
+	    } else {
+		indexes = new String[1];
+		indexes[0] = mainIndex;
+		return indexes;
+	    }
+	} else {
+	    return (String[]) getValue(ATTR_INDEXES, null) ;
+	}
+    }
+
     /**
      * Add our own Style Sheet to the HtmlGenerator.
      * @param g The HtmlGenerator.
@@ -782,51 +820,19 @@ public class HTTPFrame extends ProtocolFrame {
 	    g.addLink( new HtmlLink("STYLESHEET", css_url));
 	}
     }
-
+    
     /**
-     * Reply with an HTML doc listing the resources of this directory.
-     * This function takes special care not to regenerate a directory listing
-     * when one is available. It also caches the date of the directory 
-     * listing, so that it can win big with NOT_MODIFIED.
-     * <p>Using a modem, I know that each place I can reply with an 
-     * NOT_MODIFIED, <strong>is</strong> a big win.
-     * @param request The request to handle.
-     * @exception ProtocolException If processsing the request failed.
-     * @exception ResourceException If the resource got a fatal error.
-     */
-
-    public synchronized Reply getDirectoryListing(Request request)
-	throws ProtocolException, ResourceException
-    {
-	if (dresource == null) 
-	    throw new ResourceException("this frame is not attached to a "+
-					"DirectoryResource. ("+
-					resource.getIdentifier()+")");
-	// delete us if the directory was deleted
-	if (! dresource.getDirectory().exists()) {
-	    //delete us and emit an error
-	    String msg = dresource.getIdentifier()+
-		": deleted, removing the DirectoryResource";
-	    getServer().errlog(dresource, msg);
-	    try {
-		dresource.delete();
-	    } catch (MultipleLockException ex) {
-	    }
-	    // Emit an error back:
-	    Reply error = request.makeReply(HTTP.NOT_FOUND) ;
-	    error.setContent ("<h1>Document not found</h1>"+
-			      "<p>The document "+
-			      request.getURL()+
-			      " is indexed but not available."+
-			      "<p>The server is misconfigured.") ;
-	    throw new HTTPException (error) ;
-	}
-	// Have we already an up-to-date computed a listing ?
-	if ((listing == null) 
-	    || (dresource.getDirectory().lastModified() > listing_stamp)
-	    || (dresource.getLastModified() > listing_stamp)
-	    || (getLastModified() > listing_stamp)) {
-      
+     * Get ContainerResource listing
+     * @param refresh should we refresh the listing?
+     * @return a boolean (true if refreshed)
+     */ 
+    public synchronized boolean computeContainerListing(boolean refresh) {
+	ContainerResource cresource = (ContainerResource)resource;
+	if ((refresh) ||
+	    (listing == null) || 
+	    (cresource.getLastModified() > listing_stamp) || 
+	    (getLastModified() > listing_stamp)) {
+	    
 	    Class http_class = null;
 	    try {
 		http_class = Class.forName("org.w3c.jigsaw.frames.HTTPFrame");
@@ -834,24 +840,23 @@ public class HTTPFrame extends ProtocolFrame {
 		http_class = null;
 	    }
 
-	    Enumeration   enum      = 
-		dresource.enumerateResourceIdentifiers() ;
+	    Enumeration   enum      = cresource.enumerateResourceIdentifiers();
 	    Vector        resources = Sorter.sortStringEnumeration(enum) ;
 	    HtmlGenerator g         = 
-		new HtmlGenerator("Directory of "+
-				  dresource.getIdentifier());
+		new HtmlGenerator("Index of "+cresource.getIdentifier());
 	    // Add style link
 	    addStyleSheet(g);
-	    g.append("<h1>"+dresource.getIdentifier()+"</h1>");
+	    g.append("<h1>"+cresource.getIdentifier()+"</h1>");
 	    // Link to the parent, when possible:
-	    if ( dresource.getParent() != null )
+	    if ( cresource.getParent() != null ) {
 		g.append("<p><a href=\"..\">Parent</a><br>");
+	    }
 	    // List the children:
 	    for (int i = 0 ; i < resources.size() ; i++) {
 		String            name = (String) resources.elementAt(i);
 		ResourceReference rr   = null;
 		long              size = -1;
-		rr = dresource.lookup(name);
+		rr = cresource.lookup(name);
 		FramedResource resource = null;
 		if (rr != null) {
 		    try {
@@ -878,7 +883,7 @@ public class HTTPFrame extends ProtocolFrame {
 			    if ( icon != null ) 
 				g.append("<img src=\""+
 					 getIconDirectory() +"/" + icon+
-					 "\">");
+					 "\" alt=\"" + icon + "\">");
 			    // Resource's name with link:
 			    if (resource instanceof ContainerInterface)
 				g.append("<a href=\"" 
@@ -921,7 +926,57 @@ public class HTTPFrame extends ProtocolFrame {
 	    g.close() ;
 	    listing_stamp = getLastModified() ;
 	    listing       = g ;
-	} else if ( checkIfModifiedSince(request) == COND_FAILED ) {
+	    return true;
+	}
+	return false;
+    }
+
+    /**
+     * Reply with an HTML doc listing the resources of this container.
+     * This function takes special care not to regenerate a listing
+     * when one is available. It also caches the date of the
+     * listing, so that it can win big with NOT_MODIFIED.
+     * <p>Using a modem, I know that each place I can reply with an 
+     * NOT_MODIFIED, <strong>is</strong> a big win.
+     * @param request The request to handle.
+     * @exception ProtocolException If processsing the request failed.
+     * @exception ResourceException If the resource got a fatal error.
+     */
+
+    public synchronized Reply getDirectoryListing(Request request)
+	throws ProtocolException, ResourceException
+    {
+	if (! (resource instanceof ContainerResource)) {
+	    throw new ResourceException("this frame is not attached to a "+
+					"ContainerResource. ("+
+					resource.getIdentifier()+")");
+	}
+	// delete us if the directory was deleted
+	boolean refresh = false;
+	if (dresource != null) {
+	    if (! dresource.getDirectory().exists()) {
+		//delete us and emit an error
+		String msg = dresource.getIdentifier()+
+		    ": deleted, removing the DirectoryResource";
+		getServer().errlog(dresource, msg);
+		try {
+		    dresource.delete();
+		} catch (MultipleLockException ex) {
+		}
+		// Emit an error back:
+		Reply error = request.makeReply(HTTP.NOT_FOUND) ;
+		error.setContent ("<h1>Document not found</h1>"+
+				  "<p>The document "+
+				  request.getURL()+
+				  " is indexed but not available."+
+				  "<p>The server is misconfigured.") ;
+		throw new HTTPException (error) ;
+	    }
+	    refresh = 
+		(dresource.getDirectory().lastModified() > listing_stamp);
+	}
+	if ((! computeContainerListing(refresh)) && 
+	    ( checkIfModifiedSince(request) == COND_FAILED )) {
 	    // Is it an IMS request ?
 	    Reply reply = createDefaultReply(request, HTTP.NOT_MODIFIED) ;
 	    reply.setContentMD5(null);
@@ -965,7 +1020,9 @@ public class HTTPFrame extends ProtocolFrame {
 	    if (fresource != null) 
 		cl = fresource.getFileLength();
 	    if ( cl >= 0 ) {
-		setValue(ATTR_CONTENT_LENGTH, new Integer(cl));
+		if (cl != getInt(ATTR_CONTENT_LENGTH, -1)) {
+		    setValue(ATTR_CONTENT_LENGTH, new Integer(cl));
+		}
 		contentlength = HttpFactory.makeInteger(cl);
 	    }
 	}
@@ -1245,10 +1302,17 @@ public class HTTPFrame extends ProtocolFrame {
 		}
 		URL url = null;
 		try {
-		    url = (ls.hasRequest() 
-			   ? getURL(request)
-			   : new URL(getServer().getURL(), 
-				     resource.getURLPath()));
+		    if ((request != null ) && 
+			request.hasState(Request.ORIG_URL_STATE)) {
+			URL oldurl;
+			oldurl = (URL)request.getState(Request.ORIG_URL_STATE);
+			url = new URL(oldurl, oldurl.getFile() + "/");
+		    } else {
+			url = (ls.hasRequest() 
+			       ? getURL(request)
+			       : new URL(getServer().getURL(), 
+					 resource.getURLPath()));
+		    }
 		} catch (MalformedURLException ex) {
 		    getServer().errlog(this, "unable to build full URL.");
 		    throw new HTTPException("Internal server error");
@@ -1276,19 +1340,26 @@ public class HTTPFrame extends ProtocolFrame {
 		Request request = (Request)ls.getRequest() ;
 		request.setState(STATE_CONTENT_LOCATION, "true");
 		// return the index file.
-		String index = getIndex();
-		if ( index != null && index.length() > 0) {
-		    DirectoryResource dir = (DirectoryResource) resource;
-		    ResourceReference rr = dir.lookup(index);
-		    if (rr != null) {
-			try {
-			    FramedResource rindex = (FramedResource) rr.lock();
-			    return rindex.lookup(ls,lr);
-			} catch (InvalidResourceException ex) {
-			} finally {
-			    rr.unlock();
+		String indexes[] = getIndexes();
+		if (indexes != null) {
+		    for (int i = 0 ; i < indexes.length ; i++) {
+			String index = indexes[i];
+			if ( index != null && index.length() > 0) {
+			    DirectoryResource dir = 
+				(DirectoryResource) resource;
+			    ResourceReference rr = dir.lookup(index);
+			    if (rr != null) {
+				try {
+				    FramedResource rindex = 
+					(FramedResource) rr.lock();
+				    return rindex.lookup(ls,lr);
+				} catch (InvalidResourceException ex) {
+				} finally {
+				    rr.unlock();
+				}
+			    }
 			}
-		    }
+		    }	
 		}
 	    }
 	    return true;
@@ -1423,10 +1494,14 @@ public class HTTPFrame extends ProtocolFrame {
     protected Reply getOtherResource(Request request) 
 	throws ProtocolException, ResourceException
     {
-	// we don't manage this kind of resource
-	Reply error = request.makeReply(HTTP.NOT_IMPLEMENTED) ;
-	error.setContent("Method GET not implemented.") ;
-	throw new HTTPException (error) ;
+	if (resource instanceof ContainerResource) {
+	    return getDirectoryResource(request);
+	} else {
+	    // we don't manage this kind of resource
+	    Reply error = request.makeReply(HTTP.NOT_IMPLEMENTED) ;
+	    error.setContent("Method GET not implemented.") ;
+	    throw new HTTPException (error) ;
+	}
     }
 
     /**
