@@ -1,5 +1,5 @@
 // JpegComFrame.java
-// $Id: JpegComFrame.java,v 1.1.2.7 1999/11/16 16:36:18 ylafon Exp $
+// $Id: JpegComFrame.java,v 1.1.2.11 2000/03/09 21:11:10 ylafon Exp $
 // (c) COPYRIGHT MIT, INRIA and Keio, 1999.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -7,6 +7,7 @@ package org.w3c.jigsaw.frames;
 
 import java.util.*;
 import java.io.*;
+import java.net.URLEncoder;
 
 import org.w3c.www.mime.*;
 import org.w3c.util.*;
@@ -168,6 +169,9 @@ public class JpegComFrame extends HTTPFrame {
 	// Set the date of the reply (round it to secs):
 	reply.setDate((System.currentTimeMillis() / 1000L) * 1000L);
 	reply.setETag(getComETag());
+	String commenttype = getCommentType().toString();
+	reply.setContentLocation(getURL(request).toExternalForm() 
+				 + ";" + commenttype);
 	return reply;
     }
 
@@ -312,6 +316,30 @@ public class JpegComFrame extends HTTPFrame {
 	Reply reply = null;
 	fresource.checkContent();
 	updateCachedHeaders();
+	// hack, if ;text/html is there,
+	// it will be added at first place of the accept 
+	String param = null;
+	String sfile = request.getURL().getFile();
+	int pos = sfile.indexOf(';');
+	if (pos != -1) {
+	    param = (String) request.getState("type");
+	}
+	if (param != null) {
+	    HttpAccept acc[] = request.getAccept();
+	    HttpAccept newacc[] = null;
+	    if (acc != null) {
+		newacc = new HttpAccept[acc.length+1];
+		System.arraycopy(acc, 0, newacc, 1, acc.length);
+	    } else {
+		newacc = new HttpAccept[1];
+	    }
+	    try {
+		newacc[0] = HttpFactory.makeAccept(new MimeType(param), 1.1);
+		request.setAccept(newacc);
+	    } catch (MimeTypeFormatException ex) {
+		// not a valid mime type... maybe something else, do not care
+	    }
+	}
 	boolean commentOnly = negotiate(request);
 	HttpEntityTag etag = null;
 	if (commentOnly)
@@ -377,21 +405,29 @@ public class JpegComFrame extends HTTPFrame {
 	File file = fresource.getFile() ;
 	fresource.checkContent();
 	updateCachedHeaders();
-	String query = request.getQueryString();
-	boolean commentOnly = false;
-	// hack, if ?text/html, it will be added at first place of the accept
-	if (query != null) {
+	String param = null;
+	String sfile = request.getURL().getFile();
+	int pos = sfile.indexOf(';');
+	if (pos != -1) {
+	    param = (String) request.getState("type");
+	}
+	if (param != null) {
 	    HttpAccept acc[] = request.getAccept();
-	    HttpAccept newacc[] = new HttpAccept[acc.length+1];
-	    System.arraycopy(acc, 0, newacc, 1, acc.length);
+	    HttpAccept newacc[] = null;
+	    if (acc != null) {
+		newacc = new HttpAccept[acc.length+1];
+		System.arraycopy(acc, 0, newacc, 1, acc.length);
+	    } else {
+		newacc = new HttpAccept[1];
+	    }
 	    try {
-		newacc[0] = HttpFactory.makeAccept(new MimeType(query));
+		newacc[0] = HttpFactory.makeAccept(new MimeType(param), 1.1);
 		request.setAccept(newacc);
 	    } catch (MimeTypeFormatException ex) {
 		// not a valid mime type... maybe something else, do not care
 	    }
 	}
-	commentOnly = negotiate(request);
+	boolean commentOnly = negotiate(request);
 	HttpEntityTag etag = null;
 	if (commentOnly)
 	    etag = getComETag();
@@ -426,8 +462,6 @@ public class JpegComFrame extends HTTPFrame {
 	if ( file.exists() ) {
 	    if (commentOnly) {
 		reply = createCommentReply(request);
-		reply.setContentLocation(getURL(request).toExternalForm() 
-					 + "?" + getCommentType());
 	    } else {
 		reply = createFileReply(request);
 	    }
