@@ -1,5 +1,5 @@
 // NegotiatedFrame.java
-// $Id: NegotiatedFrame.java,v 1.29.4.1 2000/01/19 20:51:13 ylafon Exp $
+// $Id: NegotiatedFrame.java,v 1.29.4.3 2000/03/30 09:22:23 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -666,7 +666,8 @@ public class NegotiatedFrame extends HTTPFrame {
 			if ( jidx < 0 )
 			    state.setQuality (0.0) ;
 			else 
-			    state.setQuality(accepts[jidx]) ;
+			    state.setQuality(accepts[jidx].getQuality()
+					     -jidx*0.001) ;
 		    }
 		} catch (InvalidResourceException ex) {
 		    //FIXME
