@@ -1,5 +1,5 @@
 // RelocateFrame.java
-// $Id: RelocateFrame.java,v 1.12.4.1 1999/09/08 13:48:09 bmahe Exp $
+// $Id: RelocateFrame.java,v 1.12.4.2 2000/02/10 13:40:55 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -45,6 +45,10 @@ public class RelocateFrame extends HTTPFrame {
      * Attribute index - SHould we use the ambiguous 302?
      */
     protected static int ATTR_USE_302 = -1;
+    /**
+     * Attribute index - The methods affected by this frame
+     */
+    protected static int ATTR_METHODS = -1 ;
 
     static {
 	Attribute a = null ;
@@ -76,16 +80,31 @@ public class RelocateFrame extends HTTPFrame {
 				 , Boolean.TRUE
 				 , Attribute.EDITABLE);
 	ATTR_USE_302 = AttributeRegistry.registerAttribute(c, a);
+	// The affected methods
+	a = new StringArrayAttribute("methods"
+				     , null
+				     , Attribute.EDITABLE) ;
+	ATTR_METHODS = AttributeRegistry.registerAttribute(c, a) ;
     }
 
     /**
      * Get the location for the relocation
-     * @return a string, containing the relative path.
+     * @return a string, containing the relative path or absolute
      */
-
     public String getLocation() {
 	return (String) getValue(ATTR_LOCATION, null) ;
     }
+
+    /**
+     * Get the list of methods affected by the redirect
+     * @return An array of String giving the name of the redirected methods,
+     *    or <strong>null</strong>, in wich case <em>all</em> methods are
+     *    to be redirected.
+     */
+    public String[] getMethods() {
+	return (String[]) getValue(ATTR_METHODS, null) ;
+    }
+
     /**
      * Get the path info value
      * @return a boolean
@@ -93,6 +112,7 @@ public class RelocateFrame extends HTTPFrame {
     public boolean checkHandlePathInfo() {
 	return getBoolean(ATTR_HANDLE_PATHINFO, true);
     }
+
     /**
      * Get the permanent redirect flag
      * @return a boolean
@@ -100,6 +120,7 @@ public class RelocateFrame extends HTTPFrame {
     public boolean checkPermanentRedirect() {
 	return getBoolean(ATTR_PERMANENT_REDIRECT, false);
     }
+
      /**
      * Get the "use ambigous 302 response code" flag
      * @return a boolean
@@ -108,27 +129,43 @@ public class RelocateFrame extends HTTPFrame {
 	return getBoolean(ATTR_USE_302, true);
     }   
 
-    public void registerResource(FramedResource resource) {
-	super.registerOtherResource(resource);
-    }
-
-    /**
-     * Lookup the target resource when associated with an unknown resource.
+     /**
+     * Lookup the target resource (dispath to more specific lookup methods).
      * @param ls The current lookup state
      * @param lr The result
      * @return true if lookup is done.
      * @exception ProtocolException If an error relative to the protocol occurs
+     * @see #lookupDirectory
+     * @see #lookupFile
+     * @see #lookupOther
      */
-    protected boolean lookupOther(LookupState ls, LookupResult lr) 
+    protected boolean lookupResource(LookupState ls, LookupResult lr) 
 	throws ProtocolException
     {
+	String methods[] = getMethods();
+	
+	if (ls.hasRequest() && (methods != null)) {
+	    Request request = (Request) ls.getRequest();
+	    String reqmeth = request.getMethod();
+	    boolean affected = false;
+	    for (int i=0; i< methods.length; i++) {
+		if (reqmeth.equals(methods[i])) {
+		    affected = true;
+		    break;
+		}
+	    }
+	    if (!affected) {
+		return super.lookupResource(ls, lr);
+	    }
+	}
 	// Perform our super-class lookup strategy:
 	if ( super.lookupOther(ls, lr) ) {
 	    return true;
-	} else if ( ! checkHandlePathInfo() ) {
-	    return false;
-	}
-	// Compute PATH INFO, store it as a piece of state in the request:
+        } else if ( ! checkHandlePathInfo() ) {
+            return false;
+        }
+	// Compute PATH INFO, store it as a piece of state in
+	// the request:
 	StringBuffer pathinfo = new StringBuffer();
 	while ( ls.hasMoreComponents() ) {
 	    pathinfo.append('/');
@@ -136,22 +173,24 @@ public class RelocateFrame extends HTTPFrame {
 	}
 	if (ls.hasRequest() ) {
 	    Request request = (Request) ls.getRequest();
+	    String reqfile = request.getURL().getFile();
+	    if (reqfile.endsWith("/")) {
+		pathinfo.append('/');
+	    }
 	    request.setState(PATH_INFO, pathinfo.toString());
 	}
 	lr.setTarget(resource.getResourceReference());
 	return true;
     }
-
-
+    
     /**
-     * Emit a redirect.
-     * All GET requests are redirected toward the target location.
-     * @param client The client issuing the request.
+     * build the redirect reply based on the request and the current
+     * configuration
      * @param request The request to handle.
-     * @exception ProtocolException If the request couldn't be handled.
+     * @exception ProtocolException If processsing the request failed.
+     * @return a Reply
      */
-
-    protected Reply getOtherResource (Request request)
+    private Reply getRedirectReply(Request request)
 	throws ProtocolException
     {
 	String location = getLocation() ;
@@ -164,13 +203,14 @@ public class RelocateFrame extends HTTPFrame {
 	} else {
 	    Reply  reply = null;
 	    URL    loc      = null;
-	    if (checkUse302()) 
+	    if (checkUse302()) {
 		reply = request.makeReply(HTTP.FOUND) ;
-	    else {
-		if (checkPermanentRedirect()) 
+	    } else {
+		if (checkPermanentRedirect()) {
 		    reply = request.makeReply(HTTP.MOVED_PERMANENTLY) ;
-		else
+		} else {
 		    reply = request.makeReply(HTTP.TEMPORARY_REDIRECT) ;
+		}
 	    }
 	    try {
 		httpd  server = (httpd) getServer();
@@ -186,18 +226,27 @@ public class RelocateFrame extends HTTPFrame {
 		    String pathinfo = (String) request.getState(PATH_INFO);
 		    // Given the way pathinfo is computed, it starts with a /
 		    try {
-			if (pathinfo != null)
+			if (pathinfo != null) {
 			    loc = new URL(loc.toExternalForm()+pathinfo);
-			else 
-			    resource.getServer().errlog(resource, 
-					"This resource handle Pathinfo "+
-					"but the request has no "+
-					"PATH_INFO state.");
+			}
 		    } catch (MalformedURLException ex) {
 			resource.getServer().errlog(resource, 
-					"This resource handle Pathinfo "+
-					"but the request has an invalid "+
-					"PATH_INFO state.");
+					     "This resource handle Pathinfo "+
+					     "but the request has an invalid "+
+					     "PATH_INFO state.");
+		    }
+		    if (request.hasQueryString()) {
+			try {
+			    loc = new URL(loc.toExternalForm() + "?" +
+					  request.getQueryString());
+			} catch (MalformedURLException ex) {
+			    resource.getServer().errlog(resource, 
+							"This resource handle "
+							+"Pathinfo but the "
+							+"request has an " 
+							+"invalid "+
+							"PATH_INFO state.");
+			}
 		    }
 		}
 	    } catch (Exception ex) {
@@ -214,18 +263,116 @@ public class RelocateFrame extends HTTPFrame {
 	}
     }
 
-    protected Reply putOtherResource (Request request)
-	throws ProtocolException
+    /**
+     * The GET method, may emit a redirect 
+     * @param request The request to handle.
+     * @exception ProtocolException If processsing the request failed.
+     * @exception ResourceException If the resource got a fatal error.
+     */
+    public Reply get(Request request)
+	throws ProtocolException, ResourceException
     {
-	return getOtherResource(request);
+	String methods[] = getMethods();
+	
+	if (methods != null) {
+	    String reqmeth = request.getMethod();
+	    boolean affected = false;
+	    for (int i=0; i< methods.length; i++) {
+		if (reqmeth.equals(methods[i])) {
+		    affected = true;
+		    break;
+		}
+	    }
+	    if (!affected) {
+		return super.get(request);
+	    }
+	}
+	// now we can modify it :)
+	return getRedirectReply(request);
     }
 
-    protected Reply postOtherResource (Request request)
-	throws ProtocolException
+    /**
+     * The HEAD method, may emit a redirect 
+     * @param request The request to handle.
+     * @exception ProtocolException If processsing the request failed.
+     * @exception ResourceException If the resource got a fatal error.
+     */
+    public Reply head(Request request)
+	throws ProtocolException, ResourceException
     {
-	return getOtherResource(request);
+	String methods[] = getMethods();
+	
+	if (methods != null) {
+	    String reqmeth = request.getMethod();
+	    boolean affected = false;
+	    for (int i=0; i< methods.length; i++) {
+		if (reqmeth.equals(methods[i])) {
+		    affected = true;
+		    break;
+		}
+	    }
+	    if (!affected) {
+		return super.head(request);
+	    }
+	}
+	// now we can modify it :)
+	Reply reply = getRedirectReply(request);
+	reply.setStream((InputStream) null);
+	return reply ;
     }
 
-
-
+    /**
+     * The PUT method, may emit a redirect, otherwise uses its parent put
+     * @param request The request to handle.
+     * @exception ProtocolException If processsing the request failed.
+     * @exception ResourceException If the resource got a fatal error.
+     */
+    public Reply put(Request request)
+	throws ProtocolException, ResourceException
+    {
+	String methods[] = getMethods();
+	
+	if (methods != null) {
+	    String reqmeth = request.getMethod();
+	    boolean affected = false;
+	    for (int i=0; i< methods.length; i++) {
+		if (reqmeth.equals(methods[i])) {
+		    affected = true;
+		    break;
+		}
+	    }
+	    if (!affected) {
+		return super.put(request);
+	    }
+	}
+	// now we can modify it :)
+	return getRedirectReply(request);
+    }
+    /**
+     * The POST method, may emit a redirect, otherwise uses its parent put
+     * @param request The request to handle.
+     * @exception ProtocolException If processsing the request failed.
+     * @exception ResourceException If the resource got a fatal error.
+     */
+    public Reply post(Request request)
+	throws ProtocolException, ResourceException
+    {
+	String methods[] = getMethods();
+	
+	if (methods != null) {
+	    String reqmeth = request.getMethod();
+	    boolean affected = false;
+	    for (int i=0; i< methods.length; i++) {
+		if (reqmeth.equals(methods[i])) {
+		    affected = true;
+		    break;
+		}
+	    }
+	    if (!affected) {
+		return super.post(request);
+	    }
+	}
+	// now we can modify it :)
+	return getRedirectReply(request);
+    }
 }
