@@ -1,5 +1,5 @@
 // HtmlScript.java
-// $Id: HtmlScript.java,v 1.1 1998/11/04 18:48:58 bmahe Exp $
+// $Id: HtmlScript.java,v 1.1.4.1 1999/10/18 11:58:19 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -14,7 +14,7 @@ import org.w3c.www.mime.*;
  */
 
 public class HtmlScript {
-    String          lang = null;;
+    String          lang = null;
     StringBuffer    script = null;
 
     /**
