@@ -1,5 +1,5 @@
 // Client.java
-// $Id: Client.java,v 1.54.4.2 1999/11/30 15:51:31 ylafon Exp $
+// $Id: Client.java,v 1.54.4.5 2000/04/06 09:34:40 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -478,28 +478,28 @@ public abstract class Client implements EventHandler {
 			(cex.ex instanceof HttpParserException)) {
 			HttpParserException hex = (HttpParserException) cex.ex;
 			if (!hex.hasRequest()) {
-			    break alive_loop;
+			    throw (cex);
 			}
 			request = (Request) hex.getRequest();
 			if ( processed == 0 ) {
 			    major = request.getMajorVersion();
 			    minor = request.getMinorVersion();
-			} else
-			    usedConnection();
+			}
+			usedConnection();
 			reply = (Reply) request.makeBadRequestReply();
 			reply.setContentLength(0);
 			reply.addConnection("close");
 			// Inital keep alive check, emit the reply:
-			if ( keep ) 
-			    keep = tryKeepConnection(request, reply) ;
+			keep = false;
 			sent = emitReply(reply) ;
 			// Semi-fake log entry
 			log(request, reply, 0, 0) ;
 			processed++;
-			if ( keep )
-			    output.flush(); 
+			// a bad request is still a request...
+			reqcount++;
+			break alive_loop;
 		    }
-		    break alive_loop;
+		    throw (cex);
 		} catch (Exception genex) {
 		    if (debug)
 			genex.printStackTrace();
@@ -508,6 +508,7 @@ public abstract class Client implements EventHandler {
 		    if (genex.getMessage().startsWith("Bad request") && 
 			(prev_body_count > 0)) {
 			// check if the error is due to a
+			usedConnection();
 			if (debug)
 			    System.out.println("Error after a body "+
 					       "request! (Skipping: " +
@@ -516,8 +517,8 @@ public abstract class Client implements EventHandler {
 			input.reset();
 			byte b[] = new byte[(int)prev_body_count];
 			prev_body_count -= input.read(b);
-			    prev_body_count -= input.skip(prev_body_count);
-			    continue;
+			prev_body_count -= input.skip(prev_body_count);
+			break alive_loop;
 		    }
 		    throw (genex);
 		}
@@ -617,11 +618,9 @@ public abstract class Client implements EventHandler {
 		}
 	    }
 	    terminate();
-	    if (reply != null) {
-		if (reply.getStatus() != HTTP.SWITCHING)
-		    stopConnection();
-	    } else 
+	    if (reply == null || (reply.getStatus() != HTTP.SWITCHING)) {
 		stopConnection();
+	    }
 	    if ( err != null )
 		throw err;
 	}
