@@ -1,5 +1,5 @@
 // CommonLogger.java
-// $Id: CommonLogger.java,v 1.25.4.1 1999/10/12 11:46:17 ylafon Exp $
+// $Id: CommonLogger.java,v 1.25.4.7 2000/06/29 08:18:28 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -20,7 +20,7 @@ import org.w3c.util.*;
  */
 
 public class CommonLogger extends Logger implements PropertyMonitoring {
-    private static final String monthnames[] = {
+    protected static final String monthnames[] = {
 	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
     };
@@ -65,16 +65,30 @@ public class CommonLogger extends Logger implements PropertyMonitoring {
      */
     public static final 
     String BUFSIZE_P = "org.w3c.jigsaw.logger.bufferSize";
+    /**
+     * Name of the property indicating the buffer size for the logger.
+     * This buffer size applies only the the log file, not to the error
+     * log file, or the trace log file. It can be set to zero if you want
+     * no buffering.
+     * <p>This property default to <strong>4096</strong>.
+     */
+    public static final 
+    String ROTATE_LEVEL_P = "org.w3c.jigsaw.logger.rotateLevel";
 
-    private   byte                 msgbuf[] = null ;
-    protected RandomAccessFile     log      = null ;
-    protected RandomAccessFile     errlog   = null ;
-    protected RandomAccessFile     trace    = null ;
-    protected httpd                server   = null ;
-    protected ObservableProperties props    = null ;
-    protected int                  bufsize  = 8192;
-    protected int                  bufptr   = 0;
-    protected byte                 buffer[] = null;
+    private   byte                 msgbuf[]    = null ;
+    protected RandomAccessFile     log         = null ;
+    protected RandomAccessFile     errlog      = null ;
+    protected RandomAccessFile     trace       = null ;
+    protected httpd                server      = null ;
+    protected ObservableProperties props       = null ;
+    protected int                  bufsize     = 8192 ;
+    protected int                  bufptr      = 0    ;
+    protected int                  rotateLevel = 0    ;
+    protected byte                 buffer[]    = null ;
+    protected int                  year        = -1   ;
+    protected int                  month       = -1   ;
+    protected int                  day         = -1   ;
+    protected int                  tz          = -1   ;
 
     /**
      * Property monitoring for the logger.
@@ -136,11 +150,21 @@ public class CommonLogger extends Logger implements PropertyMonitoring {
 		buffer = (bufsize > 0) ? new byte[bufsize] : null;
 		return true;
 	    }
+	} else if (name.equals(ROTATE_LEVEL_P) ) {
+	    int newLevel = props.getInteger(name, rotateLevel);
+	    if (newLevel != rotateLevel) {
+		synchronized (this) {
+		    sync();
+		    rotateLevel = newLevel;
+		    openLogFile();
+		}
+	    }
+	    return true;
 	} else {
-		return true ;
+	    return true ;
 	}
     }
-
+    
     /**
      * Output the given message to the given RandomAccessFile.
      * This method makes its best effort to avoid one byte writes (which you
@@ -177,11 +201,10 @@ public class CommonLogger extends Logger implements PropertyMonitoring {
 		log.write(huge, 0, msglen);
 		return;
 	    }
-	} else {
-	    // Append that message to buffer:
-	    msg.getBytes(0, msglen, buffer, bufptr);
-	    bufptr += msglen;
-	}
+	} 
+	// Append that message to buffer:
+	msg.getBytes(0, msglen, buffer, bufptr);
+	bufptr += msglen;
     }
 
     protected void logmsg (String msg) {
@@ -222,16 +245,149 @@ public class CommonLogger extends Logger implements PropertyMonitoring {
 	}
     }
 
+    protected void checkLogFile(Date now) {
+	int nowYear = now.getYear()+1900;
+	if (rotateLevel == 1) {
+	    // rotate every year
+	    if (nowYear != year) {
+		synchronized (this) {
+		    if (log != null) {
+			sync();
+		    }
+		    this.year = nowYear;
+		    openLogFile(year);
+		}
+	    }
+	} else {
+	    int nowMonth = now.getMonth();
+	    if (rotateLevel == 2) {
+		// rotate every month
+		if ((nowYear != year) || (nowMonth != month)) {
+		    synchronized (this) {
+			if (log != null) {
+			    sync();
+			}
+			this.year = nowYear;
+			this.month = nowMonth;
+			openLogFile(year, month);
+		    }
+		}
+	    } else {
+		int nowDay = now.getDate();
+		if (rotateLevel == 3) {
+		    // rotate every day
+		    if ((nowYear != year) || (nowMonth != month) 
+			|| (nowDay != day)) {
+			synchronized (this) {
+			    if (log != null) {
+				sync();
+			    }
+			    this.year  = nowYear;
+			    this.month = nowMonth;
+			    this.day   = nowDay;
+			    openLogFile(year, month, day);
+			}
+		    } 
+		}
+	    }
+	}
+    }
+
+    protected void openLogFile(int year, int month, int day) {
+	this.year = year;
+	this.month = month;
+	this.day = day;
+
+	String ext = null;
+	if (month < 10) {
+	    if (day < 10) {
+		ext = "_"+year+"_0"+(month+1)+"_0"+day;
+	    } else {
+		ext = "_"+year+"_0"+(month+1)+"_"+day;
+	    }
+	} else {
+	    if (day < 10) {
+		ext = "_"+year+"_"+(month+1)+"_0"+day;
+	    } else {
+		ext = "_"+year+"_"+(month+1)+"_"+day;
+	    }
+	}
+	String logname = getFilename(LOGNAME_P, "log") + ext;
+	try {
+	    RandomAccessFile old = log ;
+	    log = new RandomAccessFile (logname, "rw") ;
+	    log.seek (log.length()) ;
+	    if ( old != null )
+		old.close () ;
+	} catch (IOException e) {
+	    throw new HTTPRuntimeException (this.getClass().getName()
+					    , "openLogFile"
+					    , "unable to open "+logname);
+	}
+    }
+
+    protected void openLogFile(int year, int month) {
+	this.year = year;
+	this.month = month;
+       
+	String ext = null;
+	if (month < 10)
+	    ext = "_"+year+"_0"+(month+1);
+	else
+	    ext = "_"+year+"_"+(month+1);
+
+	String logname = getFilename(LOGNAME_P, "log") + ext;
+	try {
+	    RandomAccessFile old = log ;
+	    log = new RandomAccessFile (logname, "rw") ;
+	    log.seek (log.length()) ;
+	    if ( old != null )
+		old.close () ;
+	} catch (IOException e) {
+	    throw new HTTPRuntimeException (this.getClass().getName()
+					    , "openLogFile"
+					    , "unable to open "+logname);
+	}
+    }
+
+    protected void openLogFile(int year) {
+	this.year = year;
+	
+	String logname = getFilename(LOGNAME_P, "log") + "_" + year;
+	try {
+	    RandomAccessFile old = log ;
+	    log = new RandomAccessFile (logname, "rw") ;
+	    log.seek (log.length()) ;
+	    if ( old != null )
+		old.close () ;
+	} catch (IOException e) {
+	    throw new HTTPRuntimeException (this.getClass().getName()
+					    , "openLogFile"
+					    , "unable to open "+logname);
+	}
+    }
+
     /**
      * Log the given HTTP transaction.
      * This is shamelessly slow.
      */
-
     public void log (Request request, Reply reply, int nbytes, long duration) {
 	Client client = request.getClient() ;
 	String entry  = null ;
 	long   date   = reply.getDate();
-	Date   now    = (date < 0) ? new Date() : new Date(date);
+	if ( tz == -1 ) {
+	    tz = new Date().getTimezoneOffset();
+	}
+	Date   now;
+	if (date < 0 ) {
+	    now = new Date((new Date()).getTime() + tz*60*1000);
+	} else {
+	    now = new Date(date + (tz*60*1000));
+	}
+
+	if (rotateLevel > 0) {
+	    checkLogFile(now);
+	}
 
 	String user = (String) request.getState(AuthFilter.STATE_AUTHUSER);
 
@@ -251,10 +407,7 @@ public class CommonLogger extends Logger implements PropertyMonitoring {
 	       + ((now.getSeconds() < 10)
 		  ? (":0" + now.getSeconds())
 		  : (":" + now.getSeconds()))
-	       + ((now.getTimezoneOffset() < 0)
-		  ? " " + (now.getTimezoneOffset() / 60)
-		  : " +" + (now.getTimezoneOffset() / 60))
-	       + "]")
+	       + " +0]")
 	    + " \"" + request.getMethod()	// request line
 	    + " " + request.getURL()
 	    + " " + request.getVersion()
@@ -344,17 +497,26 @@ public class CommonLogger extends Logger implements PropertyMonitoring {
      */
 
     protected void openLogFile () {
-	String logname = getFilename(LOGNAME_P, "log") ;
-	try {
-	    RandomAccessFile old = log ;
-	    log = new RandomAccessFile (logname, "rw") ;
-	    log.seek (log.length()) ;
-	    if ( old != null )
-		old.close () ;
-	} catch (IOException e) {
-	    throw new HTTPRuntimeException (this.getClass().getName()
-					    , "openLogFile"
-					    , "unable to open "+logname);
+	if (rotateLevel > 0) {
+	    if ( tz == -1 ) {
+		tz = new Date().getTimezoneOffset();
+	    }
+	    Date now = new Date();
+	    this.year = -1;
+	    checkLogFile(now);
+	} else {
+	    String logname = getFilename(LOGNAME_P, "log") ;
+	    try {
+		RandomAccessFile old = log ;
+		log = new RandomAccessFile (logname, "rw") ;
+		log.seek (log.length()) ;
+		if ( old != null )
+		    old.close () ;
+	    } catch (IOException e) {
+		throw new HTTPRuntimeException (this.getClass().getName()
+						, "openLogFile"
+						, "unable to open "+logname);
+	    }
 	}
     }
 
@@ -450,6 +612,8 @@ public class CommonLogger extends Logger implements PropertyMonitoring {
 	this.props  = server.getProperties() ;
 	// Register for property changes:
 	props.registerObserver (this) ;
+        // init the rotation level
+        rotateLevel = props.getInteger(ROTATE_LEVEL_P, 0);
 	// Open the various logs:
 	openLogFile () ;
 	openErrorLogFile() ;
