@@ -1,5 +1,5 @@
 // httpd.java
-// $Id: httpd.java,v 1.96.2.4 2000/01/21 10:12:01 ylafon Exp $
+// $Id: httpd.java,v 1.96.2.9 2000/07/04 08:53:49 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -18,6 +18,7 @@ import org.w3c.tools.timers.* ;
 import org.w3c.jigsaw.auth.*;
 import org.w3c.jigsaw.resources.*;
 import org.w3c.jigsaw.daemon.*;
+import org.w3c.jigsaw.status.Status;
 import org.w3c.www.http.*;
 import org.w3c.jigsaw.config.*;
 import org.w3c.util.*;
@@ -35,12 +36,12 @@ import org.w3c.tools.resources.ProtocolException;
  */
 
 public class httpd implements 
-      ServerInterface, Runnable, PropertyMonitoring, Cloneable
+      ServerInterface, Runnable, PropertyMonitoring, Cloneable, Status
 {
     /**
      * The current displayed version of Jigsaw.
      */
-    public static final String version = "2.0.4";
+    public static final String version = "2.0.5";
     /**
      * The current internal version counter of Jigsaw.
      * This counter is bumped anytime the configuration needs upgrade.
@@ -292,7 +293,7 @@ public class httpd implements
     
     /* FIXME */ public Thread thread    = null ;
     
-    private String         software  = "Jigsaw/2.0.4";
+    private String         software  = "Jigsaw/2.0.5";
     private ServerSocket   socket    = null ;
     private Logger         logger    = null ;
     private Shuffler       shuffler  = null ;
@@ -1254,6 +1255,20 @@ public class httpd implements
 	return new File(getConfigDirectory(), "indexers");
     }
 
+    /**
+     * Get temp directory
+     */
+    public File getTempDirectory() {
+	return new File(getRootDirectory(), "tmp");
+    }
+
+    /**
+     * Clean the temp dir.
+     */
+    protected void cleanTempDirectory() {
+	org.w3c.util.IO.clean(getTempDirectory());
+    }
+    
     /**
      * get the indexer catalog of this server 
      */
@@ -1384,6 +1399,7 @@ public class httpd implements
 	} catch (IOException ex) {
 	    errlog("[shutdown]: IOException while unblocking server thread.");
 	}
+	cleanTempDirectory();
     }
 
     /**
@@ -1866,6 +1882,22 @@ public class httpd implements
 	return rr_configResource;
     }
 
+    /**
+     * Give the status of this class as a partial HTML text which will be added
+     * into a block level element
+     * @return a String, the generated HTML
+     */
+    public String getHTMLStatus() {
+	if (factory instanceof Status) {
+	    try {
+		return ((Status)factory).getHTMLStatus();
+	    } catch (Exception ex) {
+		return getIdentifier();
+	    }
+	}
+	return getIdentifier();
+    }
+    
     /**
      * Create a new server instance in this process.
      * @param identifier The server's identifier.
