@@ -1,5 +1,5 @@
 // JigsawHttpServletRequest.java
-// $Id: JigsawHttpServletRequest.java,v 1.46 1999/06/09 09:44:21 bmahe Exp $
+// $Id: JigsawHttpServletRequest.java,v 1.46.4.12 2000/04/03 07:54:20 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -7,6 +7,7 @@ package org.w3c.jigsaw.servlet;
 
 import java.io.*;
 import java.util.*;
+import java.security.Principal;
 
 import javax.servlet.*;
 import javax.servlet.http.*;
@@ -156,7 +157,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
 	    }
 	}
 	// URL encoded parameters:
-	String query = request.getQueryString();
+	String query = getQueryString();
 	if (query != null) {
 	    Reader qis = new StringReader(query);
 	    try {
@@ -173,7 +174,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
 
     protected String getURLParameter(String name) {
 	Hashtable urlParameters = null;
-	String query = request.getQueryString();
+	String query = getQueryString();
 	if (query != null) {
 	    Reader qis = new StringReader(query);
 	    try {
@@ -203,8 +204,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return An int, or <strong>-1</strong>.
      */
 
-    public int getContentLength()
-    {
+    public int getContentLength() {
 	return request.getContentLength();
     }
 
@@ -213,8 +213,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return A String encoded mime type, or <strong>null</strong>.
      */
 
-    public String getContentType()
-    {
+    public String getContentType() {
 	org.w3c.www.mime.MimeType t = request.getContentType();
 	return (t == null) ? null : t.toString();
     }
@@ -224,24 +223,31 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return A String encoded version of the protocol.
      */
 
-    public String getProtocol()
-    {
+    public String getProtocol() {
 	return request.getVersion();
     }
     
+    protected httpd getServer() {
+	return request.getClient().getServer();
+    }
+
    /**
     * ServletRequest implementation - Get the name of queried server.
     * @return Name of server, as a String.
     */
 
-   public String getServerName()
-   {
-       if (request.isProxy()) {
-	   String host = request.getHost();
-	   if (host != null)
+   public String getServerName() {
+       String host = request.getHost();
+       if (host != null) {
+	   int idx = host.lastIndexOf(':');
+	   if (idx != -1) {
 	       return host.substring(0, host.lastIndexOf(':'));
+	   } else {
+	       return host;
+	   }
+       } else {
+	   return getServer().getHost();
        }
-       return request.getClient().getServer().getHost();
    }
 
    /**
@@ -249,8 +255,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
     * @return A port number (int).
     */
 
-   public int getServerPort()
-   {
+   public int getServerPort() {
        if (request.isProxy()) {
 	   String host = request.getHost();
 	   if (host != null) {
@@ -260,7 +265,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
 	       return Integer.parseInt(host.substring(idx+1));
 	   }
        }
-       return request.getClient().getServer().getLocalPort();
+       return getServer().getLocalPort();
    }
 
    /**
@@ -268,8 +273,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
     * @return Numeric IP address, as a String.
     */
 
-    public String getRemoteAddr()
-    {
+    public String getRemoteAddr() {
 	return request.getClient().getInetAddress().getHostAddress();
     }
 
@@ -278,8 +282,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return Name of client's machine (FQDN).
      */
 
-    public String getRemoteHost()
-    {
+    public String getRemoteHost() {
 	return request.getClient().getInetAddress().getHostName();
     }
 
@@ -292,46 +295,11 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      */
 
     public String getRealPath(String name) {
-	httpd             server  = request.getClient().getServer();
+	httpd             server  = getServer();
 	ResourceReference rr_root = server.getRootReference();
-	try {
-	    FramedResource root = (FramedResource)rr_root.lock();
-	    if (root instanceof VirtualHostResource) {
-		//backward to the virtual host resource
-		ResourceReference rr  = request.getTargetResource();
-		ResourceReference rrp = null;
-		while (true) {
-		    try {
-			FramedResource res = (FramedResource)rr.lock();
-			rrp = res.getParent();
-			if ((rrp == rr_root) || (rrp == null)) {
-			    if (res.definesAttribute("directory")) {
-				File dir = 
-				    (File)res.getValue("directory", null);
-				if (dir != null)
-				    return (new File(dir, name)).
-					getAbsolutePath();
-			    }
-			    return null;
-			}
-		    } catch (InvalidResourceException ex) {
-			return null;
-		    } finally {
-			rr.unlock();
-		    }
-		    rr = rrp;
-		}
-	    } else if (root.definesAttribute("directory")) {
-		File dir = (File)root.getValue("directory", null);
-		if (dir != null)
-		    return (new File(dir, name)).getAbsolutePath();
-	    }
-	    return null;
-	} catch (InvalidResourceException ex) {
-	    return null;
-	} finally {
-	    rr_root.unlock();
-	}
+	return JigsawServletContext.getRealPath(name, 
+						rr_root, 
+						request.getTargetResource());
     }
 
     protected ServletInputStream is = null;
@@ -373,8 +341,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return The String encoded value for the parameter.
      */
 
-    public String getParameter(String name)
-    {
+    public String getParameter(String name) {
 	prepareQueryParameters();
 	if ( queryParameters != null ) {
 	    Object value = queryParameters.get(name);
@@ -412,8 +379,7 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return An enumeration of parameter names.
      */
     
-    public Enumeration getParameterNames()
-    {
+    public Enumeration getParameterNames() {
 	prepareQueryParameters();
 	return ((queryParameters == null)
 		? new EmptyEnumeration()
@@ -435,17 +401,118 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
 	request.setState(name, object);
     }
 
+    /**
+     * Removes an attribute from this request.  This method is not
+     * generally needed as attributes only persist as long as the request
+     * is being handled.
+     *
+     * <p>Attribute names should follow the same conventions as
+     * package names. Names beginning with <code>java.*</code>,
+     * <code>javax.*</code>, and <code>com.sun.*</code>, are
+     * reserved for use by Sun Microsystems.
+     *
+     * @param name a <code>String</code> specifying 
+     * the name of the attribute to remove
+     */
+    public void removeAttribute(String name) {
+	request.delState(name);
+    }
+
     public Enumeration getAttributeNames() {
 	return request.getStateNames();
     }
 
+    /**
+     * Returns the preferred <code>Locale</code> that the client will 
+     * accept content in, based on the Accept-Language header.
+     * If the client request doesn't provide an Accept-Language header,
+     * this method returns the default locale for the server.
+     *
+     * @return the preferred <code>Locale</code> for the client
+     */
+    public Locale getLocale() {
+	return (Locale)getLocales().nextElement();
+    }
+    
+    /**
+     * Returns an <code>Enumeration</code> of <code>Locale</code> objects
+     * indicating, in decreasing order starting with the preferred locale, the
+     * locales that are acceptable to the client based on the Accept-Language
+     * header.
+     * If the client request doesn't provide an Accept-Language header,
+     * this method returns an <code>Enumeration</code> containing one 
+     * <code>Locale</code>, the default locale for the server.
+     *
+     * @return an <code>Enumeration</code> of preferred 
+     * <code>Locale</code> objects for the client
+     */
+    public Enumeration getLocales() {
+	HttpAcceptLanguage languages[] = request.getAcceptLanguage();
+	if (languages == null) {
+	    Vector def = new Vector();
+            def.addElement(Locale.getDefault());
+            return def.elements();
+	}
+
+	//LinkedList is better, but we must be JDK1.1 compliant
+	Vector locales = new Vector(); 
+
+	for (int i = 0 ; i < languages.length ; i++) {
+	    HttpAcceptLanguage language = languages[i];
+	    double quality = language.getQuality();
+	    String lang    = language.getLanguage();
+	    String country = "";
+	    int    idx     = lang.indexOf('-');
+	    if (idx > -1) {
+		country = lang.substring(idx + 1).trim();
+		lang    = lang.substring(0, idx).trim();
+	    }
+	    // insert the Locale in ordered list
+	    int     qidx = 0;
+	    int     size = locales.size();
+	    if (size > 0) {
+		QLocale ql   = (QLocale) locales.firstElement();
+		while ((qidx < size) && (ql.getLanguageQuality() >= quality)) {
+		    try {
+			ql = (QLocale) locales.elementAt(++qidx);
+		    } catch (ArrayIndexOutOfBoundsException ex) {
+			//end of vector, so append
+		    }
+		}
+		locales.insertElementAt(new QLocale(lang, country, quality),
+					qidx);
+	    } else {
+		locales.addElement(new QLocale(lang, country, quality));
+	    }
+	}
+	// because Locale is final :(
+	int    size    = locales.size(); 
+	Vector vlocale = new Vector(size);
+	for (int i = 0 ; i < size ; i ++) {
+	    vlocale.addElement(((QLocale)locales.elementAt(i)).getLocale());
+	}
+	return vlocale.elements();
+    }
+
+    /**
+     * Returns a boolean indicating whether this request was made using a
+     * secure channel, such as HTTPS.
+     *
+     * @return a boolean indicating if the request was made using a
+     * secure channel
+     */
+
+    public boolean isSecure() {
+	// only https secure?
+	return (request.getURL().getProtocol().equalsIgnoreCase("https"));
+    }
+    
     /**
      * HttpServletRequest implementation - Get the request's method.
      * @return A String instance.
      */
 
-    public  String getMethod()
-    {
+    public  String getMethod() {
 	return request.getMethod();
     }
 
@@ -454,9 +521,13 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return A String instance or <strong>null</strong>.
      */
 
-    public  String getPathInfo()
-    {
-	return (String) request.getState(ServletWrapperFrame.STATE_EXTRA_PATH);
+    public  String getPathInfo() {
+	if (request.hasState(JigsawRequestDispatcher.PATH_INFO_P)) {
+	    String pathinfo = 
+		(String) request.getState(JigsawRequestDispatcher.PATH_INFO_P);
+	    return (pathinfo.equals("/")) ? null : pathinfo;
+	}
+	return null;
     }
     
     /**
@@ -464,22 +535,46 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return A String instance or <strong>null</strong>.
      */
     
-    public  String getPathTranslated()
-    {
+    public  String getPathTranslated() {
 	String pathinfo = getPathInfo();
 	if ( pathinfo != null )
 	    return getRealPath(pathinfo);
 	return null;
     }
 
+    /**
+     * Returns the portion of the request URI that indicates the context
+     * of the request.  The context path always comes first in a request
+     * URI.  The path starts with a "/" character but does not end with a "/"
+     * character.  For servlets in the default (root) context, this method
+     * returns "".
+     * @return a <code>String</code> specifying the portion of the request 
+     * URI that indicates the context of the request
+     */
+    public String getContextPath() {
+	throw new RuntimeException("Not Yet Implemented");
+    }
+
+    public boolean hasQueryString() {
+	if (request.hasQueryString()) {
+	    return true;
+	} else {
+	    return request.hasState(JigsawRequestDispatcher.QUERY_STRING_P);
+	}
+    }
+
     /**
      * HttpServletRequest implementation - Get the request's query string.
      * @return A String instance or <strong>null</strong>.
      */
-
-    public  String getQueryString()
-    {
-	return request.getQueryString();
+    public String getQueryString() {
+	if (request.hasQueryString()) {
+	    return request.getQueryString();
+	} else if (request.hasState(JigsawRequestDispatcher.QUERY_STRING_P)) {
+	    return (String) 
+		request.getState(JigsawRequestDispatcher.QUERY_STRING_P);
+	}
+	return null;
     }
     
     /**
@@ -487,11 +582,39 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * @return A String instance or <strong>null</strong>.
      */
 
-    public String getRemoteUser()
-    {
+    public String getRemoteUser() {
 	return (String) request.getState(AuthFilter.STATE_AUTHUSER);
     }
     
+    /**
+     * Returns a boolean indicating whether the authenticated user is included
+     * in the specified logical "role".  Roles and role membership can be
+     * defined using deployment descriptors.  If the user has not been
+     * authenticated, the method returns <code>false</code>.
+     *
+     * @param role a <code>String</code> specifying the name of the role
+     * @return a <code>boolean</code> indicating whether the user making this
+     * request belongs to a given role; <code>false</code> if the user has not
+     * been authenticated
+     */
+
+    public boolean isUserInRole(String role) {
+	throw new RuntimeException("Not Yet Implemented");
+    }
+
+    /**
+     * Returns a <code>java.security.Principal</code> object containing
+     * the name of the current authenticated user. If the user has not been
+     * authenticated, the method returns <code>null</code>.
+     *
+     * @return a <code>java.security.Principal</code> containing
+     * the name of the user making this request; <code>null</code> if the 
+     * user has not been authenticated
+     */
+    public Principal getUserPrincipal() {
+	return new PrincipalImpl(getRemoteUser());
+    }
+
     /**
      * HttpServletRequest implementation - Get the request's auth method.
      * @return A String instance or <strong>null</strong>.
@@ -510,6 +633,36 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
 	return request.getValue(name);
     }
 
+    /**
+     * Returns all the values of the specified request header
+     * as an <code>Enumeration</code> of <code>String</code> objects.
+     *
+     * <p>Some headers, such as <code>Accept-Language</code> can be sent
+     * by clients as several headers each with a different value rather than
+     * sending the header as a comma separated list.
+     *
+     * <p>WARNING, this can't happen with Jigsaw, all multiple values are
+     * grouped in one, and only one, header. So, this method always return 
+     * ONE header value.
+     *
+     * <p>If the request did not include any headers
+     * of the specified name, this method returns an empty
+     * <code>Enumeration</code>.
+     * The header name is case insensitive. You can use
+     * this method with any request header.
+     *
+     * @param name a <code>String</code> specifying the header name
+     * @return a <code>Enumeration</code> containing the values of the 
+     * requested header, or <code>null</code> if the request does not
+     * have any headers of that name
+     */			
+
+    public Enumeration getHeaders(String name) {
+	String value = getHeader(name);
+	String array[] = { value };
+	return new ArrayEnumeration(array);
+    }
+
     /**
      * HttpServletRequest implementation - Get a request header as an int.
      * @return An int, or <strong>-1</strong>.
@@ -553,36 +706,100 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
      * Gets, from the first line of the HTTP request, 
      * the part of this request's URI that is to the left of any query string.
      */
-    public  String getRequestURI()
-    {
+    public  String getRequestURI() {
 	String uri = null;
-	//fixme test
-	if (request.isProxy()) {
-	    uri = request.getURL().toExternalForm();
+	if (request.hasState(JigsawRequestDispatcher.REQUEST_URI_P)) {
+	    uri = (String)
+		request.getState(JigsawRequestDispatcher.REQUEST_URI_P);
 	} else {
-	    uri = request.getURLPath();
-	}
-	if (request.hasQueryString()) {
-	    String query = request.getQueryString();
-	    int idx = uri.lastIndexOf(query);
-	    uri = uri.substring(0, idx-1);
+	    //fixme test
+	    if (request.isProxy()) {
+		uri = request.getURL().toExternalForm();
+	    } else {
+		uri = request.getURLPath();
+	    }
+	    if (hasQueryString()) {
+		String query = getQueryString();
+		int idx = uri.lastIndexOf(query);
+		uri = uri.substring(0, idx-1);
+	    }
 	}
 	return uri;
     }
     
+    /**
+     * Returns a {@link RequestDispatcher} object that acts as a wrapper for
+     * the resource located at the given path.  
+     * A <code>RequestDispatcher</code> object can be used to forward
+     * a request to the resource or to include the resource in a response.
+     * The resource can be dynamic or static.
+     *
+     * <p>The pathname specified may be relative, although it cannot extend
+     * outside the current servlet context.  If the path begins with 
+     * a "/" it is interpreted as relative to the current context root.  
+     * This method returns <code>null</code> if the servlet container
+     * cannot return a <code>RequestDispatcher</code>.
+     *
+     * <p>The difference between this method and {@link
+     * ServletContext#getRequestDispatcher} is that this method can take a
+     * relative path.
+     *
+     * @param path a <code>String</code> specifying the pathname
+     * to the resource
+     * @return a <code>RequestDispatcher</code> object that acts as a 
+     * wrapper for the resource at the specified path
+     * @see RequestDispatcher
+     * @see ServletContext#getRequestDispatcher
+     */
+    public RequestDispatcher getRequestDispatcher(String path) {
+	if (path == null) {
+	    throw new IllegalArgumentException("null");
+	}
+	String            urlpath = null;
+	ResourceReference rr      = request.getTargetResource();
+	if (! path.startsWith("/")) {
+	    String uri = null;
+	    try {
+		ResourceReference rrp = rr.lock().getParent();
+		try {
+		    Resource r = rrp.lock();
+		    uri = r.getURLPath();
+		} catch (InvalidResourceException irex) {
+		    return null;
+		} finally {
+		    rrp.unlock();
+		}
+	    } catch (InvalidResourceException ex) {
+		return null;
+	    } finally {
+		rr.unlock();
+	    }
+	    urlpath = ( uri.endsWith("/") ? uri+path : uri+"/"+path );
+	} else {
+	    urlpath = path;
+	}
+	return JigsawRequestDispatcher.getRequestDispatcher(urlpath, 
+							    getServer(),
+							    rr);
+    }
+
     /**
      * Gets the part of this request's URI that refers to the servlet 
      * being invoked. Analogous to the CGI variable SCRIPT_NAME. 
      */
-    public  String getServletPath()
-    {
-	ResourceReference rr = request.getTargetResource();
-	try {
-	    return rr.lock().getURLPath();
-	} catch (InvalidResourceException ex) {
-	    return null;
-	} finally {
-	    rr.unlock();
+    public String getServletPath() {
+	if (request.hasState(JigsawRequestDispatcher.SERVLET_PATH_P)) {
+	    return (String)
+		request.getState(JigsawRequestDispatcher.SERVLET_PATH_P);
+	} else {
+	    ResourceReference rr = request.getTargetResource();
+	    try {
+		return rr.lock().getURLPath();
+	    } catch (InvalidResourceException ex) {
+		return null;
+	    } finally {
+		rr.unlock();
+	    }
 	}
     }
     
@@ -834,7 +1051,8 @@ public class JigsawHttpServletRequest implements HttpServletRequest {
     }
 
     /**
-     * @@FIXME
+     * Get the wrapped Jigsaw Request.
+     * @return the request
      */
     protected Request getRequest() {
 	return request;
