@@ -1,5 +1,5 @@
 // JigsawHttpServletReponse.java
-// $Id: JigsawHttpServletResponse.java,v 1.31.4.1 1999/11/29 16:49:53 bmahe Exp $
+// $Id: JigsawHttpServletResponse.java,v 1.31.4.9 2000/03/14 17:50:50 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -9,10 +9,12 @@ import java.io.*;
 import javax.servlet.*;
 import javax.servlet.http.*;
 import java.net.*;
+import java.util.Locale;
 
 import org.w3c.www.mime.*;
 import org.w3c.www.http.*;
 import org.w3c.jigsaw.http.*;
+import org.w3c.www.http.HeaderValue;
 
 /**
  * @author Alexandre Rafalovitch <alex@access.com.au>
@@ -23,9 +25,15 @@ import org.w3c.jigsaw.http.*;
 
 public class JigsawHttpServletResponse implements HttpServletResponse {
 
+    public final static String CHARSET_PARAMETER = "charset";
+
+    public final static int DEFAULT_BUFFER_SIZE   = 8 * 1024; // 8KB buffer
+    public final static int MIN_BUFFER_SIZE       = 4 * 1024; // 4KB buffer
+
     private final static int STATE_INITIAL        = 0;
     private final static int STATE_HEADERS_DONE   = 1;
     private final static int STATE_ALL_DONE       = 2;
+
     private final static int STREAM_STATE_INITIAL = 0;
     private final static int STREAM_WRITER_USED   = 1;
     private final static int OUTPUT_STREAM_USED   = 2;
@@ -42,6 +50,9 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
     private final static int CALC_CONTENT_LENGTH = -1; 
     private              int fixedContentLength  = CALC_CONTENT_LENGTH;
 
+    // Our Locale
+    protected Locale locale;
+
     /**
      * Our temp stream.
      */
@@ -49,6 +60,8 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
 
     protected JigsawHttpServletRequest jrequest = null;
 
+    protected int buffer_size;
+
     protected void setServletRequest(JigsawHttpServletRequest jrequest) {
 	this.jrequest = jrequest;
     }
@@ -116,10 +129,10 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
 	if (stream_state == STREAM_WRITER_USED)
 	    throw new IllegalStateException("Writer used");
 	stream_state = OUTPUT_STREAM_USED;
-	return getJigsawOutputStream();
+	return getJigsawOutputStream(false);
     }
 
-    protected ServletOutputStream getJigsawOutputStream()
+    protected ServletOutputStream getJigsawOutputStream(boolean writerUsed)
 	throws IOException
     {
 	if ( output != null )
@@ -128,16 +141,21 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
 	// any exception during setContentType ?    
 	if( setContentTypeException != null ) {
 	    // "wrap" the exception from setContentType in an IOException
-	    throw new IOException("Illegal Contenttype: "+
+	    throw new IOException("Illegal Content Type: "+
 				  setContentTypeException.toString());
 	}
 
 	if( request.hasState(INCLUDED) ) {
 	    out = new ByteArrayOutputStream();
 	    output = new JigsawServletOutputStream(this, 
-						   new DataOutputStream(out));
+						   new DataOutputStream(out),
+						   buffer_size,
+						   writerUsed);
 	} else {
-	    output = new JigsawServletOutputStream(this, reply);
+	    output = new JigsawServletOutputStream(this, 
+						   reply, 
+						   buffer_size,
+						   writerUsed);
 	}
 	return output;
     }
@@ -259,7 +277,12 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
 	URL loc = null;
 	try {
 	    loc = new URL(request.getURL(), url);
-	    setStatus(SC_MOVED_TEMPORARILY);
+	    if (jrequest.getMethod().equalsIgnoreCase("POST") &&
+		jrequest.getProtocol().equals("HTTP/1.1")) {
+		setStatus(SC_SEE_OTHER);
+	    } else {
+		setStatus(SC_MOVED_TEMPORARILY);
+	    }
 	    reply.setLocation(loc);
 	    state = STATE_ALL_DONE;
 	} catch (Exception ex) {
@@ -407,8 +430,8 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
      */
     public String getCharacterEncoding() {
 	org.w3c.www.mime.MimeType type = reply.getContentType();
-	if ((type != null) && (type.hasParameter("charset"))) {
-	    return type.getParameterValue("charset");
+	if ((type != null) && (type.hasParameter(CHARSET_PARAMETER))) {
+	    return type.getParameterValue(CHARSET_PARAMETER);
 	}
 	return System.getProperty("file.encoding");
     }
@@ -436,7 +459,7 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
       
 	if (writer == null) {
 	    writer = new PrintWriter(
-			 new OutputStreamWriter(getJigsawOutputStream(), 
+			 new OutputStreamWriter(getJigsawOutputStream(true), 
 						getCharacterEncoding()));
 	}
 	return writer;
@@ -450,12 +473,21 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
     protected synchronized void flushStream(boolean close) 
 	throws IOException
     {
+	if (state == STATE_ALL_DONE) {
+	  return;
+	}
 	int writeLength;
 
 	if (stream_state == OUTPUT_STREAM_USED) {
 	    output.flush();
 	} else if (stream_state == STREAM_WRITER_USED) {
 	    writer.flush();
+	    output.realFlush();
+	} else {
+	    // force flush even if no stream are openned
+	    getWriter();
+	    writer.flush();
+	    output.realFlush();
 	}
 
 	if (request.hasState(INCLUDED)) {
@@ -482,9 +514,231 @@ public class JigsawHttpServletResponse implements HttpServletResponse {
 	}
     }
 
+    // 2.2
+
+    /**
+     * Sets the preferred buffer size for the body of the response.  
+     * The servlet container will use a buffer at least as large as 
+     * the size requested.  The actual buffer size used can be found
+     * using <code>getBufferSize</code>.
+     *
+     * <p>A larger buffer allows more content to be written before anything is
+     * actually sent, thus providing the servlet with more time to set
+     * appropriate status codes and headers.  A smaller buffer decreases 
+     * server memory load and allows the client to start receiving data more
+     * quickly.
+     *
+     * <p>This method must be called before any response body content is
+     * written; if content has been written, this method throws an 
+     * <code>IllegalStateException</code>.
+     * @param size the preferred buffer size
+     * @exception IllegalStateException	if this method is called after
+     * content has been written
+     * @see #getBufferSize
+     * @see #flushBuffer
+     * @see #isCommitted
+     * @see #reset
+     */
+    public void setBufferSize(int size) {
+	if (stream_state != STREAM_STATE_INITIAL) {
+	    throw new IllegalStateException("Stream already initialized");
+	}
+	buffer_size = size < MIN_BUFFER_SIZE ? MIN_BUFFER_SIZE : size;
+    }
+
+    /**
+     * Returns the actual buffer size used for the response.  If no buffering
+     * is used, this method returns 0.
+     * @return the actual buffer size used
+     * @see #setBufferSize
+     * @see #flushBuffer
+     * @see #isCommitted
+     * @see #reset
+     */
+    public int getBufferSize() {
+	return buffer_size;
+    }
+
+    /**
+     * Forces any content in the buffer to be written to the client.  A call
+     * to this method automatically commits the response, meaning the status 
+     * code and headers will be written.
+     * @see #setBufferSize
+     * @see #getBufferSize
+     * @see #isCommitted
+     * @see #reset
+     */
+    public void flushBuffer() 
+	throws IOException
+    {
+	if (output != null) {
+	    if (stream_state == STREAM_WRITER_USED) {
+		writer.flush();
+	    }
+	    output.flush();
+	}
+    }
+
+    /**
+     * Returns a boolean indicating if the response has been
+     * committed.  A commited response has already had its status 
+     * code and headers written.
+     * @return a boolean indicating if the response has been
+     * committed
+     * @see #setBufferSize
+     * @see #getBufferSize
+     * @see #flushBuffer
+     * @see #reset
+     */
+    public boolean isCommitted() {
+	if (output != null) {
+	    return output.isCommitted();
+	} else {
+	    return false;
+	}
+    }
+
+    /**
+     * Clears any data that exists in the buffer as well as the status code and
+     * headers.  If the response has been committed, this method throws an 
+     * <code>IllegalStateException</code>.
+     * @exception IllegalStateException  if the response has already been
+     *                                   committed
+     * @see #setBufferSize
+     * @see #getBufferSize
+     * @see #flushBuffer
+     * @see #isCommitted
+     */
+    public void reset() {
+	if (output != null) {
+	    if (stream_state == STREAM_WRITER_USED) {
+		writer.flush();
+	    }
+	    output.reset();
+	}
+    }
+
+    /**
+     * Sets the locale of the response, setting the headers (including the
+     * Content-Type's charset) as appropriate.  This method should be called
+     * before a call to {@link #getWriter}.  By default, the response locale
+     * is the default locale for the server.
+     * @param loc  the locale of the response
+     * @see #getLocale
+     */
+    public void setLocale(Locale locale) {
+	if (locale == null) {
+	    return;
+	}
+	this.locale = locale;
+
+	// content language
+	String lang = locale.getLanguage();
+	if (lang.length() > 0) {
+	    String array[] = new String[1];
+	    array[0] = lang;
+	    reply.setContentLanguage(array);
+	}
+	
+	// content type (charset)
+	String charset = org.w3c.www.mime.Utils.getCharset(locale);
+	if (charset != null) {
+	    MimeType contentType = reply.getContentType();
+	    // override charset
+	    contentType.setParameter(CHARSET_PARAMETER, charset);
+	}
+    }
+
+    /**
+     * Returns the locale assigned to the response.
+     * @see #setLocale
+     */
+    public Locale getLocale() {
+	return locale;
+    }
+
+    public void addHeader(String name, String value) {
+	String      lname = name.toLowerCase();
+	HeaderValue hvalue = reply.getHeaderValue(lname);
+	//
+	// Horrible, Shame on us, I hate that
+	//
+	if (hvalue == null) {
+	    setHeader(name, value);
+	} else if (hvalue instanceof HttpAcceptCharsetList) {
+	    HttpAcceptCharsetList acl = (HttpAcceptCharsetList) hvalue;
+	    acl.addCharset(HttpFactory.parseAcceptCharset(value));
+	} else if (hvalue instanceof HttpAcceptEncodingList) {
+	    HttpAcceptEncodingList ael = (HttpAcceptEncodingList) hvalue;
+	    ael.addEncoding(HttpFactory.parseAcceptEncoding(value));
+	} else if (hvalue instanceof HttpAcceptLanguageList) {
+	    HttpAcceptLanguageList all = (HttpAcceptLanguageList) hvalue;
+	    all.addLanguage(HttpFactory.parseAcceptLanguage(value));
+	} else if (hvalue instanceof HttpAcceptList) {
+	    HttpAcceptList al = (HttpAcceptList) hvalue;
+	    al.addAccept(HttpFactory.parseAccept(value));
+	} else if (hvalue instanceof HttpEntityTagList) {
+	    HttpEntityTagList etl = (HttpEntityTagList) hvalue;
+	    etl.addTag(HttpFactory.parseETag(value));
+	} else if (hvalue instanceof HttpExtList) {
+	    HttpExtList el = (HttpExtList) hvalue;
+	    el.addHttpExt(new HttpExt(value, false));
+	} else if (hvalue instanceof HttpCookieList) {
+	    // shouldn't be used, but who knows?
+	    HttpCookieList cl  = (HttpCookieList) hvalue;
+	    HttpCookieList ncl = HttpFactory.parseCookieList(value);
+	    HttpCookie scookies[] = ncl.getCookies();
+	    for (int i = 0 ; i < scookies.length ; i++) {
+		HttpCookie cookie = scookies[i];
+		cl.addCookie(cookie.getName(), cookie.getValue());
+	    }
+	} else if (hvalue instanceof HttpParamList) {
+	    int idx = value.indexOf('=');
+	    if (idx != -1) {
+		String pname  = value.substring(0, idx);
+		String pvalue = value.substring(idx+1);
+		HttpParamList pl = (HttpParamList) hvalue;
+		pl.setParameter(pname, pvalue);
+	    }
+	} else if (hvalue instanceof HttpRangeList) {
+	    HttpRangeList rl = (HttpRangeList) hvalue;
+	    rl.addRange(HttpFactory.parseRange(value));
+	} else if (hvalue instanceof HttpSetCookieList) {
+	    HttpSetCookieList scl = (HttpSetCookieList) hvalue;
+	    HttpSetCookieList nscl = HttpFactory.parseSetCookieList(value);
+	    HttpSetCookie scookies[] = nscl.getSetCookies();
+	    for (int i = 0 ; i < scookies.length ; i++) {
+		scl.addSetCookie(scookies[i]);
+	    }
+	} else if (hvalue instanceof HttpTokenList) {
+	    ((HttpTokenList) hvalue).addToken(value, true);
+	} else if (hvalue instanceof HttpWarningList) {
+	    HttpWarningList wl = (HttpWarningList) hvalue;
+	    wl.addWarning(HttpFactory.parseWarning(value));
+	} else if (hvalue instanceof HttpString) {
+	    // this is the default type for unkown header
+	    // we don't know what it is, so just append
+	    HttpString s = (HttpString) hvalue;
+	    String string = (String) s.getValue();
+	    s.setValue(string+", "+value);
+	} else {
+	    // not compliant with HTTP/1.1, override
+	    setHeader(name, value);
+	}
+    }
+
+    public void addDateHeader(String name, long date) {
+	addHeader(name, HttpFactory.makeDate(date).toExternalForm());
+    }
+
+    public void addIntHeader(String name, int value) {
+	addHeader(name, String.valueOf(value));
+    }
+
     JigsawHttpServletResponse(Request request, Reply reply) {
-	this.request = request;
-	this.reply   = reply;
+	this.request     = request;
+	this.reply       = reply;
+	this.buffer_size = DEFAULT_BUFFER_SIZE;
     }
 
 }
