@@ -1,5 +1,5 @@
 // JigsawRequestDispatcher.java
-// $Id: JigsawRequestDispatcher.java,v 1.2.4.1 1999/10/25 08:37:38 bmahe Exp $
+// $Id: JigsawRequestDispatcher.java,v 1.2.4.4 2000/03/14 17:50:50 bmahe Exp $
 // (c) COPYRIGHT MIT, INRIA and Keio, 1999.
 // Please first read the full copyright statement in file COPYRIGHT.html
  
@@ -24,7 +24,9 @@ import org.w3c.jigsaw.http.httpd;
 import org.w3c.jigsaw.http.Request;
 import org.w3c.jigsaw.http.Reply;
 
+import org.w3c.tools.resources.InvalidResourceException;
 import org.w3c.tools.resources.ResourceReference;
+import org.w3c.tools.resources.Resource;
 import org.w3c.tools.resources.LookupState;
 import org.w3c.tools.resources.LookupResult;
 import org.w3c.tools.resources.FramedResource;
@@ -34,11 +36,22 @@ import org.w3c.tools.resources.ProtocolException;
 import org.w3c.www.http.HTTP;
 
 /**
- * @version $Revision: 1.2.4.1 $
+ * @version $Revision: 1.2.4.4 $
  * @author  Benot Mah (bmahe@w3.org)
  */
 public class JigsawRequestDispatcher implements RequestDispatcher {
     
+    public static final String REQUEST_URI_P = 
+	"javax.servlet.include.request_uri";
+    public static final String CONTEXT_PATH_P = 
+	"javax.servlet.include.context_path";
+    public static final String SERVLET_PATH_P = 
+	"javax.servlet.include.servlet_path";
+    public static final String PATH_INFO_P = 
+	"javax.servlet.include.path_info";
+    public static final String QUERY_STRING_P = 
+	"javax.servlet.include.query_string";
+
     private httpd  server  = null;
     private String urlpath = null;
     
@@ -125,6 +138,11 @@ public class JigsawRequestDispatcher implements RequestDispatcher {
 
 	Reply reply = null;
 	try {
+	    //req.setState(CONTEXT_PATH_P, jreq.getContextPath());
+	    req.setState(REQUEST_URI_P, jreq.getRequestURI());
+	    req.setState(SERVLET_PATH_P, jreq.getServletPath());
+	    req.setState(PATH_INFO_P, jreq.getPathInfo());
+	    req.setState(QUERY_STRING_P, jreq.getQueryString());
 	    req.setState(JigsawHttpServletResponse.INCLUDED, Boolean.TRUE);
 	    reply = (Reply) server.perform(req);
 	} catch (ResourceException ex) {
@@ -163,29 +181,75 @@ public class JigsawRequestDispatcher implements RequestDispatcher {
 	this.urlpath = urlpath;
     }
 
+    /**
+     * Get the appropriate dispatcher
+     * @param name The servlet name
+     * @param rr the ServletContainer (ServletDirectoryFrame) reference
+     * @param server the HTTP server
+     * @return the RequestDispatcher
+     */
+    public static RequestDispatcher getRequestDispatcher(String name,
+							 ResourceReference rr,
+							 httpd server)
+    {
+	try {
+	    Resource res = rr.lock();
+	    if (! (res instanceof ServletDirectoryFrame)) {
+		throw new IllegalArgumentException("Not a servlet container!");
+	    }
+	    ServletDirectoryFrame sdf = (ServletDirectoryFrame) res;
+	    if (sdf.getServlet(name) != null) {
+		// servlet exists, so return the dispatcher
+		String urlpath = sdf.getResource().getURLPath();
+		urlpath = 
+		    urlpath.endsWith("/") ? urlpath+name : urlpath+"/"+name;
+		return new JigsawRequestDispatcher(urlpath, server);
+	    }
+	} catch(InvalidResourceException ex) {
+	    return null;
+	} finally {
+	    rr.unlock();
+	}
+	return null;
+    }
+
+    /**
+     * Get the appropriate dispatcher
+     * @param urlpath the servlet URI
+     * @param server the HTTP server
+     * @param rr the ServletContainer reference or a servlet reference 
+     * (Just used for Virtual Host)
+     * @return the RequestDispatcher
+     */
     public static RequestDispatcher getRequestDispatcher(String urlpath,
-							 httpd server) 
+							 httpd server,
+							 ResourceReference rr) 
     {
 	ResourceReference rr_root = null;
-	rr_root = server.getRootReference();
-
-	FramedResource root = null;
-	root = server.getRoot();
-
-	// Do the lookup:
-	ResourceReference r_target = null;
+	FramedResource    root = null;
+	rr_root = JigsawServletContext.getLocalRoot(server.getRootReference(),
+						    rr);
 	try {
-	    LookupState  ls = new LookupState(urlpath);
-	    LookupResult lr = new LookupResult(rr_root);
-	    root.lookup(ls, lr);
-	    r_target = lr.getTarget();
-	} catch (Exception ex) {
-	    r_target = null;
-	}
-	if (r_target == null)
+	    root = (FramedResource) rr_root.lock();
+	    // Do the lookup:
+	    ResourceReference r_target = null;
+	    try {
+		LookupState  ls = new LookupState(urlpath);
+		LookupResult lr = new LookupResult(rr_root);
+		root.lookup(ls, lr);
+		r_target = lr.getTarget();
+	    } catch (Exception ex) {
+		r_target = null;
+	    }
+	    if (r_target == null)
+		return null;
+	    //there is a resource, return the dispatcher...
+	    return new JigsawRequestDispatcher(urlpath, server);
+	} catch (InvalidResourceException ex) {
 	    return null;
-	//there is a resource, return the dispatcher...
-	return new JigsawRequestDispatcher(urlpath, server);
+	} finally {
+	    rr_root.unlock(); 
+	}
     }
 
 }
