@@ -1,5 +1,5 @@
 // JigsawServletContext.java
-// $Id: JigsawServletContext.java,v 1.24 1999/08/09 08:39:45 bmahe Exp $
+// $Id: JigsawServletContext.java,v 1.24.4.6 2000/03/14 17:50:50 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1998.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -16,11 +16,12 @@ import org.w3c.tools.resources.store.*;
 import org.w3c.tools.resources.event.*;
 import org.w3c.tools.resources.*;
 import org.w3c.jigsaw.frames.*;
+import org.w3c.jigsaw.proxy.ForwardFrame;
 import org.w3c.jigsaw.resources.VirtualHostResource;
 import org.w3c.jigsaw.http.httpd;
 
 /**
- * @version $Revision: 1.24 $
+ * @version $Revision: 1.24.4.6 $
  * @author  Benot Mah (bmahe@w3.org)
  */
 public class JigsawServletContext extends StructureChangedAdapter
@@ -28,6 +29,8 @@ public class JigsawServletContext extends StructureChangedAdapter
 					     PropertyMonitoring
 {
 
+    public static final String TEMPDIR_P = "javax.servlet.context.tempdir";
+
     class Logger {
 	File             logfile  = null;
 	RandomAccessFile log      = null ;
@@ -136,9 +139,8 @@ public class JigsawServletContext extends StructureChangedAdapter
 	if (name.equals(ServletProps.SERVLET_LOG_FILE_P)) {
 	    if (logger != null) {
 		logger.close();
-		File newlogfile = 
-		    new File((String) props.get(
-					    ServletProps.SERVLET_LOG_FILE_P));
+		File newlogfile = new File((String) 
+				   props.get(ServletProps.SERVLET_LOG_FILE_P));
 		if (newlogfile.getPath().length() < 1) 
 		    newlogfile = getServletLogFile();
 		logger = new Logger(newlogfile);
@@ -281,11 +283,94 @@ public class JigsawServletContext extends StructureChangedAdapter
 
     /**
      * ServletContext implementation - Translate a piece of path.
+     * @param path the virtual path to translate
+     * @param rr_root the Root ResourceReference
+     * @param rr the target ResourceReference
+     * @return the real path
      */
+    protected static String getRealPath(String path, 
+					ResourceReference rr_root,
+					ResourceReference rr) 
+    {
+	ResourceReference local_root = getLocalRoot(rr_root, rr);
+	try {
+	    FramedResource root = (FramedResource)local_root.lock();
+	    LookupState    ls   = new LookupState(path);
+	    LookupResult   lr   = new LookupResult(local_root);
+	    if (root.lookup(ls,lr)) {
+		ResourceReference  target = lr.getTarget();
+		if (target != null) {
+		    try {
+			FramedResource res = (FramedResource)target.lock();
+			if (res instanceof FileResource) {
+			    File file = ((FileResource)res).getFile();
+			    return file.getAbsolutePath();
+			} else if (res instanceof DirectoryResource) {
+			    DirectoryResource dir = (DirectoryResource) res;
+			    return dir.getDirectory().getAbsolutePath();
+			    //return getFilePath(dir);
+			}
+		    } finally {
+			target.unlock();
+		    }
+		}
+	    }
+	    return null;
+	} catch (InvalidResourceException ex) {
+	    return null;
+	} catch (org.w3c.tools.resources.ProtocolException pex) {
+	    return null;
+	} finally {
+	    local_root.unlock();
+	}
+    }
 
+    /**
+     * ServletContext implementation - Translate a piece of path.
+     * @param path the virtual path to translate
+     * @return the real path
+     */
     public String getRealPath(String path) {
-	httpd             server  = (httpd)getServer();
-	ResourceReference rr_root = server.getRootReference();
+	ResourceReference rr_root = ((httpd) getServer()).getRootReference();
+	return getRealPath(path, rr_root, reference);
+    }
+
+    protected static String getFilePath(DirectoryResource dir) {
+	HTTPFrame frame = 
+	    (HTTPFrame)dir.getFrame("org.w3c.jigsaw.frames.HTTPFrame");
+	String indexes[] = frame.getIndexes();
+	if (indexes != null) {
+	    for (int i = 0 ; i < indexes.length ; i++) {
+		String index = indexes[i];
+		if ( index != null && index.length() > 0) {
+		    ResourceReference rr = dir.lookup(index);
+		    if (rr != null) {
+			try {
+			    FramedResource ri = (FramedResource) rr.lock();
+			    if (ri instanceof FileResource) {
+				FileResource fr = (FileResource) ri;
+				File file = fr.getFile();
+				return file.getAbsolutePath();
+			    } else {
+				// we don't know 
+				return null;
+			    }
+			} catch (InvalidResourceException ex) {
+			} finally {
+			    rr.unlock();
+			}
+		    }
+		}
+	    }
+	    return dir.getDirectory().getAbsolutePath();
+	} else {
+	    return dir.getDirectory().getAbsolutePath();
+	}
+    }
+
+    protected static ResourceReference getLocalRoot(ResourceReference rr_root,
+						    ResourceReference ref) 
+    {
 	try {
 	    FramedResource root = (FramedResource)rr_root.lock();
 	    if (root instanceof VirtualHostResource) {
@@ -294,13 +379,17 @@ public class JigsawServletContext extends StructureChangedAdapter
 		ResourceReference rrp = null;
 		FramedResource    res = null;
 		try {
-		    res = (FramedResource)reference.lock();
-		    ServletDirectoryFrame fr = (ServletDirectoryFrame)res;
-		    rr = fr.getResource().getResourceReference();
+		    res = (FramedResource)ref.lock();
+		    if (res instanceof ResourceFrame) {
+			ResourceFrame fr = (ResourceFrame)res;
+			rr = fr.getResource().getResourceReference();
+		    } else {
+			rr = ref;
+		    }
 		} catch (InvalidResourceException ex) {
-		    return null;
+		    return rr_root;
 		} finally {
-		    reference.unlock();
+		    ref.unlock();
 		}
 
 		while (true) {
@@ -308,30 +397,32 @@ public class JigsawServletContext extends StructureChangedAdapter
 			res = (FramedResource)rr.lock();
 			rrp = res.getParent();
 			if ((rrp == rr_root) || (rrp == null)) {
-			    if (res.definesAttribute("directory")) {
-				File dir = 
-				    (File)res.getValue("directory", null);
-				if (dir != null)
-				    return (new File(dir, path)).
-					getAbsolutePath();
-			    }
-			    return null;
+			    return getLocalRoot(rr, ref);
 			}
 		    } catch (InvalidResourceException ex) {
-			return null;
+			return rr_root;
 		    } finally {
 			rr.unlock();
 		    }
 		    rr = rrp;
 		}
-	    } else if (root.definesAttribute("directory")) {
-		File dir = (File)root.getValue("directory", null);
-		if (dir != null)
-		    return (new File(dir, path)).getAbsolutePath();
+	    } else {
+		try {
+		    FramedResource res = (FramedResource)rr_root.lock();
+		    ForwardFrame   ffr = (ForwardFrame)
+			res.getFrame("org.w3c.jigsaw.proxy.ForwardFrame");
+		    if (ffr == null) {
+			return rr_root;
+		    } else {
+			ResourceReference rr = ffr.getLocalRootResource();
+			return getLocalRoot(rr, ref);
+		    }
+		} catch (InvalidResourceException ex) {
+		    return rr_root;
+		}
 	    }
-	    return null;
 	} catch (InvalidResourceException ex) {
-	    return null;
+	    return rr_root;
 	} finally {
 	    rr_root.unlock();
 	}
@@ -389,6 +480,45 @@ public class JigsawServletContext extends StructureChangedAdapter
 	return attributes.keys();
     }
 
+    /**
+     * Returns a <code>String</code> containing the value of the named
+     * context-wide initialization parameter, or <code>null</code> if the 
+     * parameter does not exist.
+     *
+     * <p>This method can make available configuration information useful
+     * to an entire "web application".  For example, it can provide a 
+     * webmaster's email address or the name of a system that holds 
+     * critical data.
+     *
+     * @param name a <code>String</code> containing the name of the
+     * parameter whose value is requested
+     * @return 	a <code>String</code> containing at least the 
+     * servlet container name and version number
+     * @see ServletConfig#getInitParameter
+     */
+    public String getInitParameter(String name) {
+	// @@ not implemented @@
+	return null;
+    }
+    
+    
+
+
+    /**
+     * Returns the names of the context's initialization parameters as an
+     * <code>Enumeration</code> of <code>String</code> objects, or an
+     * empty <code>Enumeration</code> if the context has no initialization
+     * parameters.
+     *
+     * @return an <code>Enumeration</code> of <code>String</code> 
+     * objects containing the names of the context's initialization parameters
+     * @see ServletConfig#getInitParameter
+     */
+    public Enumeration getInitParameterNames() {
+	// @@ not implemented @@
+	return new EmptyEnumeration();
+    }
+
     private AutoReloadServletLoader loader = null;
 
     /** 
@@ -430,9 +560,39 @@ public class JigsawServletContext extends StructureChangedAdapter
      * @param urlpath Path to use to look up the target server resource
      */
     public RequestDispatcher getRequestDispatcher(String urlpath) {
-	return 
-	    JigsawRequestDispatcher.getRequestDispatcher(urlpath, 
-							 (httpd)getServer());
+	return JigsawRequestDispatcher.getRequestDispatcher(urlpath, 
+							   (httpd)getServer(),
+							    reference);
+    }
+
+    /**
+     * Returns a {@link RequestDispatcher} object that acts
+     * as a wrapper for the named servlet.
+     *
+     * <p>Servlets (and JSP pages also) may be given names via server 
+     * administration or via a web application deployment descriptor.
+     * A servlet instance can determine its name using 
+     * {@link ServletConfig#getServletName}.
+     *
+     * <p>This method returns <code>null</code> if the 
+     * <code>ServletContext</code>
+     * cannot return a <code>RequestDispatcher</code> for any reason.
+     *
+     * @param name a <code>String</code> specifying the name
+     * of a servlet to wrap
+     * @return a <code>RequestDispatcher</code> object
+     * that acts as a wrapper for the named servlet
+     * @see RequestDispatcher
+     * @see ServletContext#getContext
+     * @see ServletConfig#getServletName
+     */
+    public RequestDispatcher getNamedDispatcher(String name) {
+	if (name == null) {
+	    throw new IllegalArgumentException("null");
+	}
+	return JigsawRequestDispatcher.getRequestDispatcher(name, 
+							    reference,
+							   (httpd)getServer());
     }
 
     public int getMajorVersion() {
@@ -440,7 +600,7 @@ public class JigsawServletContext extends StructureChangedAdapter
     }
 
     public int getMinorVersion() {
-	return 1;
+	return 2;
     }
     
     public ServletContext getContext(String uripath) {
@@ -485,36 +645,24 @@ public class JigsawServletContext extends StructureChangedAdapter
     }
 
     public URL getResource(String path) 
-        throws MalformedURLException
+	throws MalformedURLException
     {
-        if (path.length() > 0 && path.charAt(0) == '/')
-            path = path.substring(1);
-        if (File.separatorChar != '/')
-            path.replace('/', File.separatorChar);
-        String dir = getServletDirectory().getAbsolutePath();
-        File file = new File(dir, path);
-        if (! file.exists())
-            return null;
-        String url = file.getAbsolutePath();
-        if (File.separatorChar != '/')
-            url = url.replace(File.separatorChar, '/');
-        if (url.length() > 0 && url.charAt(0) != '/')
-            url = '/' + url;
-        return new URL("file", "", url);
+	String realpath = getRealPath(path);
+	if (realpath != null) {
+	    File file = new File(realpath);
+	    if (file.exists()) {
+		return new URL("file", "", file.getAbsolutePath());
+	    } else {
+		return null;
+	    }
+	} else {
+	    // it could be a virtual resource
+	    // check that it exists (on server)
+	    // FIXME (Virtual host)
+	    return null;
+	}
     }
 
-//     public URL getResource(String path) 
-// 	throws MalformedURLException
-//     {
-// 	String url = getServletDirectory().getAbsolutePath();
-// 	if ( File.separatorChar != '/' )
-// 	    url = url.replace(File.separatorChar, '/');
-// 	File file = new File(url,path);
-// 	if (! file.exists())
-// 	    return null;
-// 	return new URL("file", "localhost", url+"/"+path);
-//     }
-
     public InputStream getResourceAsStream(String path) {
 	try {
 	    URL resource = getResource(path);
@@ -549,8 +697,8 @@ public class JigsawServletContext extends StructureChangedAdapter
 	try {
 	    Resource res = reference.lock();
 	    if (! (res instanceof ServletDirectoryFrame)) {
-		throw new RuntimeException("This reference is not pointing on"+
-					   " a ServletDirectoryFrame.");
+		throw new IllegalArgumentException("This reference is not "+
+				      "pointing on a ServletDirectoryFrame.");
 	    } else {
 		ServletDirectoryFrame sframe = (ServletDirectoryFrame)res;
 		FramedResource resource = (FramedResource)sframe.getResource();
@@ -560,8 +708,8 @@ public class JigsawServletContext extends StructureChangedAdapter
 			(File) resource.getValue("directory", null);
 	    }
 	} catch(InvalidResourceException ex) {
-	    throw new RuntimeException("This reference is pointing on"+
-				       " an Invalid ServletDirectoryFrame.");
+	    throw new IllegalArgumentException("This reference is pointing on"+
+				         " an Invalid ServletDirectoryFrame.");
 	} finally {
 	    reference.unlock();
 	}
