@@ -1,5 +1,5 @@
 // Ssiframe.java
-// $Id: SSIFrame.java,v 1.9 1998/10/15 08:15:52 bmahe Exp $
+// $Id: SSIFrame.java,v 1.11.4.1 2000/03/10 09:31:47 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -635,7 +635,7 @@ public class SSIFrame extends HTTPFrame {
 	char ch ;
 	for(int i=0;i<max;i++) {
 	    ch = (char) unparsed[start+i];
-	    if(Character.isSpace(ch)) break ;
+	    if(Character.isWhitespace(ch)) break ;
 	    cmdBuf.append(ch) ;
 	}
 	return cmdBuf.length() == 0? null : cmdBuf.toString() ;
@@ -758,6 +758,7 @@ public class SSIFrame extends HTTPFrame {
     {
 	Reply reply = super.createDefaultReply(request,status) ;
 	reply.setHeaderValue(Reply.H_LAST_MODIFIED,null) ;
+	reply.setKeepConnection(false);
 	return reply ;
     }
 
