@@ -1,12 +1,12 @@
 // Jpeg.java
-// $Id: Jpeg.java,v 1.1.2.2 1999/09/29 12:31:13 ylafon Exp $
+// $Id: Jpeg.java,v 1.1.2.3 2000/03/31 13:18:39 ylafon Exp $
 // (c) COPYRIGHT MIT, INRIA and Keio, 1999.
 // Please first read the full copyright statement in file COPYRIGHT.html
  
 package org.w3c.tools.jpeg;
 
 /**
- * @version $Revision: 1.1.2.2 $
+ * @version $Revision: 1.1.2.3 $
  * @author  Benot Mah (bmahe@w3.org)
  */
 public interface Jpeg {
@@ -40,12 +40,26 @@ public interface Jpeg {
 
     /* Start Of Scan (begins compressed data) */
     public int M_SOS   = 0xDA;	
-
+    
+    
     /* Application-specific marker, type N */
     public int M_APP0  = 0xE0;	
-
-    /* (we don't bother to list all 16 APPn's) */
+    public int M_APP1  = 0xE1;
+    public int M_APP2  = 0xE2;
+    public int M_APP3  = 0xE3;
+    public int M_APP4  = 0xE4;
+    public int M_APP5  = 0xE5;
+    public int M_APP6  = 0xE6;
+    public int M_APP7  = 0xE7;
+    public int M_APP8  = 0xE8;
+    public int M_APP9  = 0xE9;
+    public int M_APP10 = 0xEA;
+    public int M_APP11 = 0xEB;
     public int M_APP12 = 0xEC;	
+    public int M_APP13 = 0xED;
+    public int M_APP14 = 0xEE;
+    public int M_APP15 = 0xEF;
+
     public int M_COM   = 0xFE;
     /* The maximal comment length */
     public int M_MAX_COM_LENGTH = 65500;
