@@ -1,5 +1,5 @@
 // DirectoryResource.java
-// $Id: DirectoryResource.java,v 1.11 1998/12/01 09:59:49 bmahe Exp $
+// $Id: DirectoryResource.java,v 1.13.4.2 2000/02/24 10:49:22 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -128,10 +128,10 @@ public class DirectoryResource extends ContainerResource {
      * This handles the <code>RESOURCE_REMOVED</code> kind of events.
      * @param evt The event describing the change.
      */
-    
     public void resourceRemoved(StructureChangedEvent evt) {
 	super.resourceRemoved(evt);
-	markModified();
+	if (! isUnloaded())
+	    markModified();
     }
 
 
@@ -269,7 +269,7 @@ public class DirectoryResource extends ContainerResource {
 		rr_indexer = getIndexer(context);
 		context = context.getParent();
 	    } while ((rr_indexer == rr_lastidx) && (context != null));
-	    // Is this a usefull indexer ?
+	    // Is this a useful indexer ?
 	    if ((rr_lastidx = rr_indexer) != null ) {
 		try {
 		    ResourceIndexer indexer = 
@@ -379,8 +379,9 @@ public class DirectoryResource extends ContainerResource {
 
     /**
      * Reindex recursivly all the resources from this DirectoryResource.
+     * @param rec recursivly?
      */
-    public synchronized void reindex() {
+    public synchronized void reindex(boolean rec) {
 	if (getExtensibleFlag()) {
 	    Enumeration       e    = enumerateAllResourceIdentifiers();
 	    String            name = null;
@@ -405,8 +406,8 @@ public class DirectoryResource extends ContainerResource {
 			    //indexing ...
 			    Resource newdir = index(name, defs);
 			    // do we want it to keep its indexer?
-			    if (newdir == null) {
-				dir.reindex();
+			    if ((newdir == null) && rec) {
+				dir.reindex(true);
 			    } else {
 				if (! (newdir instanceof DirectoryResource)) {
 				    throw new RuntimeException(
@@ -420,13 +421,17 @@ public class DirectoryResource extends ContainerResource {
 				String indexer = 
 				    reindexed.getString(ATTR_INDEXER, "");
 				if (indexer.equals("")) {
-				    dir.reindex();
+				    if (rec) {
+					dir.reindex(true);
+				    }
 				    indexer = 
 					dir.getString(ATTR_INDEXER, null);
 				    reindexed.setValue(ATTR_INDEXER, indexer);
 				} else {
 				    dir.setValue(ATTR_INDEXER, indexer);
-				    dir.reindex();
+				    if (rec) {
+					dir.reindex(true);
+				    }
 				}
 				//move children to the reindexed directory
 				reindexed.setValue(ATTR_KEY, dir.getKey());
