@@ -1,5 +1,5 @@
 // TreeBrowser.java
-// $Id: TreeBrowser.java,v 1.12 1998/10/26 14:27:50 ylafon Exp $ */
+// $Id: TreeBrowser.java,v 1.12.4.1 1999/10/18 11:58:19 bmahe Exp $ */
 // Authors: Jean-Michel.Leon@sophia.inria.fr, 
 //          Yves.Lafon@w3.org : 
 //          - Lines, insert/remove, awt 1.1 version
@@ -120,7 +120,7 @@ public class TreeBrowser extends Canvas implements AdjustmentListener {
     private int startx = 0;
     private Color selectColor = new Color(0, 0, 128);
     private Color selectFontColor = Color.white;
-    private int scrollbarDisplayPolicy = SCROLLBARS_ASNEEDED;;
+    private int scrollbarDisplayPolicy = SCROLLBARS_ASNEEDED;
     private boolean hierarchyChanged = true;
     
     protected Vector items;
