@@ -1,5 +1,5 @@
 // IO.java
-// $Id: IO.java,v 1.2 1999/09/02 14:04:17 bmahe Exp $
+// $Id: IO.java,v 1.2.4.2 2000/03/22 18:12:30 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1998.
 // Please first read the full copyright statement in file COPYRIGHT.html
  
@@ -8,7 +8,7 @@ package org.w3c.util;
 import java.io.*;
 
 /**
- * @version $Revision: 1.2 $
+ * @version $Revision: 1.2.4.2 $
  * @author  Benot Mah (bmahe@w3.org)
  */
 public class IO {
@@ -50,5 +50,43 @@ public class IO {
 	bout.close();
 	bin.close();
     } 
-    
+
+    /**
+     * Delete recursivly
+     * @param file the file (or directory) to delete.
+     */
+    public static boolean delete(File file) {
+	if (file.exists()) {
+	    if (file.isDirectory()) {
+		if (clean(file)) {
+		    return file.delete();
+		} else {
+		    return false;
+		}
+	    } else {
+		return file.delete();
+	    }
+	}
+	return true;
+    }
+
+    /**
+     * Clean recursivly
+     * @param file the directory to clean
+     */
+    public static boolean clean(File file) {
+	if (file.isDirectory()) {
+	    String filen[] = file.list();
+	    for (int i = 0 ; i < filen.length ; i++) {
+		File subfile = new File(file, filen[i]);
+		if ((subfile.isDirectory()) && (! clean(subfile))) {
+		    return false;
+		} else if (! subfile.delete()) {
+		    return false;
+		}
+	    }
+	}
+	return true;
+    }
+
 }
