@@ -1,5 +1,5 @@
 // HttpMessage.java
-// $Id: HttpMessage.java,v 1.36.4.2 1999/12/02 18:06:14 ylafon Exp $
+// $Id: HttpMessage.java,v 1.36.4.4 2000/05/03 15:51:03 bmahe Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -1440,13 +1440,149 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
 	return emitdate;
     }
 
+    /**
+     * Get an Http Extension Listfrom the following list:
+     * <ul>
+     * <li>Mandatory (End-to-End) Extensions
+     * <li>Mandatory (Hop-by-Hop) Extensions
+     * <li>Optionnal (End-to-End) Extensions
+     * <li>Optionnal (hop-by-Hop) Extensions
+     * </ul>
+     * @param ext the Extension id
+     * @return a HttpExtList containing the extension declaration (or null)
+     */
+    public HttpExtList getExtList(String id) {
+	HttpExtList extlist = getHttpManExtDecl();
+	HttpExt ext = (extlist != null ? extlist.getHttpExt(id) : null);
+	if (ext == null) {
+	    extlist = getHttpCManExtDecl();
+	    ext = (extlist != null ? extlist.getHttpExt(id) : null);
+	    if (ext == null) {
+		extlist = getHttpOptExtDecl();
+		ext = (extlist != null ? extlist.getHttpExt(id) : null);
+		if (ext == null) {
+		    extlist = getHttpCOptExtDecl();
+		    ext = (extlist != null ? extlist.getHttpExt(id) : null);
+		    if (ext == null) {
+			extlist = null;
+		    }
+		}
+	    }
+	}
+	return extlist;
+    }
+
+    /**
+     * get the String value of the given header relative to the given 
+     * Extension. This method search the header in the following declarations:
+     * <ul>
+     * <li>Mandatory (End-to-End) Extensions
+     * <li>Mandatory (Hop-by-Hop) Extensions
+     * <li>Optionnal (End-to-End) Extensions
+     * <li>Optionnal (hop-by-Hop) Extensions
+     * </ul>
+     * @param ext the extension
+     * @param header the extension header
+     * @return a String (or null)
+     */
+    public String getExtHeader(String ext, String header) {
+	String value = getManExtHeader(ext, header);
+	if (value == null) {
+	    value = getCManExtHeader(ext, header);
+	    if (value == null) {
+		value = getOptExtHeader(ext, header);
+		if (value == null) {
+		    value = getCOptExtHeader(ext, header);
+		}
+	    }
+	}
+	return value;
+    }
+
+
+    /**
+     * get the String value of the given header relative to the given 
+     * Extension. This method search the header in the following declarations:
+     * <ul>
+     * <li>Mandatory (End-to-End) Extensions
+     * </ul>
+     * @param ext the extension
+     * @param header the extension header
+     * @return a String (or null)
+     */
+    public String getManExtHeader(String ext, String header) {
+	return getExtHeader(getHttpManExtDecl(), ext, header);
+    }
+
+    /**
+     * get the String value of the given header relative to the given 
+     * Extension. This method search the header in the following declarations:
+     * <ul>
+     * <li>Optionnal (End-to-End) Extensions
+     * </ul>
+     * @param ext the extension
+     * @param header the extension header
+     * @return a String (or null)
+     */
+    public String getOptExtHeader(String ext, String header) {
+	return getExtHeader(getHttpOptExtDecl(), ext, header);
+    }
+
+    /**
+     * get the String value of the given header relative to the given 
+     * Extension. This method search the header in the following declarations:
+     * <ul>
+     * <li>Mandatory (Hop-by-Hop) Extensions
+     * </ul>
+     * @param ext the extension
+     * @param header the extension header
+     * @return a String (or null)
+     */
+    public String getCManExtHeader(String ext, String header) {
+	return getExtHeader(getHttpCManExtDecl(), ext, header);
+    }
+
+    /**
+     * get the String value of the given header relative to the given 
+     * Extension. This method search the header in the following declarations:
+     * <ul>
+     * <li>Optionnal (hop-by-Hop) Extensions
+     * </ul>
+     * @param ext the extension
+     * @param header the extension header
+     * @return a String (or null)
+     */
+    public String getCOptExtHeader(String ext, String header) {
+	return getExtHeader(getHttpCOptExtDecl(), ext, header);
+    }
+
+    private String getExtHeader(HttpExtList list, String ext, String header) 
+    {
+	if (list == null) {
+	    return null;
+	}
+	HttpExt extheader = list.getHttpExt(ext);
+	if (extheader != null) {
+	    String realheader = extheader.getNamespace()+"-"+header;
+	    return getValue(realheader);
+	}
+	return null;
+    }
+
     /**
      * Get The Mandatory (End-to-End) Extension declaration list.
      * @return a HttpExtList instance or null;
      */
     public HttpExtList getHttpManExtDecl() {
 	HeaderValue value = getHeaderValue(H_MAN_EXT);
-	return (value != null) ? (HttpExtList) value.getValue() : null;
+	HttpExtList list  = null;
+	if (value != null) {
+	    list = (HttpExtList) value.getValue();
+	    if (list != null) {
+		list.setManOptFlag(HttpExtList.MAN);
+	    }
+	}
+	return list;
     }
 
     /**
@@ -1454,6 +1590,7 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
      * @param exts the extension declaration list.
      */
     public void setHttpManExtDecl(HttpExtList exts) {
+	exts.setManOptFlag(HttpExtList.MAN);
 	setNamespaces(exts);
 	setHeaderValue(H_MAN_EXT, exts);
     }
@@ -1464,7 +1601,40 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
      */
     public HttpExtList getHttpCManExtDecl() {
 	HeaderValue value = getHeaderValue(H_CMAN_EXT);
-	return (value != null) ? (HttpExtList) value.getValue() : null;
+	HttpExtList list  = null;
+	if (value != null) {
+	    list  = (HttpExtList) value.getValue();
+	    if (list != null) {
+		list.setManOptFlag(HttpExtList.CMAN);
+	    }
+	}
+	return list;
+    }
+
+    /**
+     * Set the Extension declaration.
+     * WARNING: The ManOpt flag of exts must have been set.
+     * @param exts the extension declaration list.
+     */
+    public void setHttpExtDecl(HttpExtList exts) {
+	setNamespaces(exts);
+	switch (exts.getManOptFlag()) 
+	    {
+	    case HttpExtList.MAN:
+		setHeaderValue(H_MAN_EXT, exts);
+		break;
+	    case HttpExtList.CMAN:
+		setHeaderValue(H_CMAN_EXT, exts);
+		break;
+	    case HttpExtList.OPT:
+		setHeaderValue(H_OPT_EXT, exts);
+		break;
+	    case HttpExtList.COPT:
+		setHeaderValue(H_COPT_EXT, exts);
+		break;
+	    default:
+		setHeaderValue(H_MAN_EXT, exts);
+	    }
     }
 
     /**
@@ -1472,6 +1642,7 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
      * @param exts the extension declaration list.
      */
     public void setHttpCManExtDecl(HttpExtList exts) {
+	exts.setManOptFlag(HttpExtList.CMAN);
 	setNamespaces(exts);
 	setHeaderValue(H_CMAN_EXT, exts);
     }
@@ -1482,7 +1653,14 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
      */
     public HttpExtList getHttpOptExtDecl() {
 	HeaderValue value = getHeaderValue(H_OPT_EXT);
-	return (value != null) ? (HttpExtList) value.getValue() : null;
+	HttpExtList list  = null;
+	if (value != null) {
+	    list = (HttpExtList) value.getValue();
+	    if (list != null) {
+		list.setManOptFlag(HttpExtList.OPT);
+	    }
+	}
+	return list;
     }
 
     /**
@@ -1490,6 +1668,7 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
      * @param exts the extension declaration list.
      */
     public void setHttpOptExtDecl(HttpExtList exts) {
+	exts.setManOptFlag(HttpExtList.OPT);
 	setNamespaces(exts);
 	setHeaderValue(H_OPT_EXT, exts);
     }
@@ -1500,7 +1679,14 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
      */
     public HttpExtList getHttpCOptExtDecl() {
 	HeaderValue value = getHeaderValue(H_COPT_EXT);
-	return (value != null) ? (HttpExtList) value.getValue() : null;
+	HttpExtList list  = null;
+	if (value != null) {
+	    list = (HttpExtList) value.getValue();
+	    if (list != null) {
+		list.setManOptFlag(HttpExtList.COPT);
+	    }
+	}
+	return list;
     }
 
     /**
@@ -1508,6 +1694,7 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
      * @param exts the extension declaration list.
      */
     public void setHttpCOptExtDecl(HttpExtList exts) {
+	exts.setManOptFlag(HttpExtList.COPT);
 	setNamespaces(exts);
 	setHeaderValue(H_COPT_EXT, exts);
     }
@@ -1590,6 +1777,10 @@ public class HttpMessage implements MimeHeaderHolder, Cloneable, HTTP {
 	setValue("C-Ext","");
     }
 
+    public final static void registerExtHeader(String name, String cls) {
+	registerHeader(name, cls);
+    }
+
     public HttpMessage(MimeParser parser) {
 	// FIXME
 	this.values  = new HeaderValue[MAX_HEADERS];
