@@ -1,5 +1,5 @@
 // HttpReplyMessage.java
-// $Id: HttpReplyMessage.java,v 1.26 1999/03/17 18:06:17 bmahe Exp $
+// $Id: HttpReplyMessage.java,v 1.26.4.1 2000/02/08 18:22:23 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -407,8 +407,8 @@ public class HttpReplyMessage extends HttpEntityMessage {
 
     /**
      * Set the location value of the reply.
-     * @param location The location, or <strong>null</strong> to reset the 
-     * value.
+     * @param location The location an URL instance, or <strong>null</strong>
+     * to reset the value.
      */
 
     public void setLocation(URL location) {
@@ -418,6 +418,19 @@ public class HttpReplyMessage extends HttpEntityMessage {
 			  : new HttpString(true, location.toExternalForm())));
     }
 
+    /**
+     * Set the location value of the reply.
+     * @param location The location a String, or <strong>null</strong> to 
+     * reset the value.
+     */
+
+    public void setLocation(String location) {
+	setHeaderValue(H_LOCATION
+		       , ((location == null)
+			  ? null
+			  : new HttpString(true, location)));
+    }
+
     /**
      * Get the proxy authentication challenge from this reply.
      * @return An instance of HttpChallenge, or <strong>null</strong> 
