@@ -1,5 +1,5 @@
 // HttpBasicServer.java
-// $Id: HttpBasicServer.java,v 1.31.4.1 1999/11/30 10:30:22 ylafon Exp $
+// $Id: HttpBasicServer.java,v 1.31.4.6 2000/03/25 01:20:24 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -87,9 +87,60 @@ public class HttpBasicServer extends HttpServer {
     short   minor     = -1;
     boolean keepalive = false;
 
-    InetAddress addrs[] = null;
-    int         addrptr = 0;
+    InetAddress addrs[]     = null;
+    int         addrptr     = 0;
+    Date        lookupLimit = null;
 
+    /**
+     * set the inet addresses of this host, and timestamp it
+     * to avoid caching IPs forever
+     * @param hostAddrs, an array of InetAddress
+     */
+    protected void setHostAddr(InetAddress hostAddrs[]) {
+	this.addrs = hostAddrs;
+	// FIXME now defaults to 300s (5mn) but needs to be a parameter
+	lookupLimit = new Date(System.currentTimeMillis() + 300 * 1000 );
+    }
+
+    /**
+     * check the validity of the host address
+     * if invalid, it will update the InetAddress associated with this host
+     */
+    protected void updateHostAddr()
+	throws HttpException
+    {
+	Date now = new Date();
+	if ((lookupLimit == null) || lookupLimit.before(now)) {
+	    // Get this server IP addreses:
+	    try {
+		InetAddress hostAddrs[];
+		// Is the given string a valid IP address ?
+		int     hostlen  = host.length();
+		boolean isstring = true;
+		for (int i = 0 ; i < hostlen ; i++) {
+		    char c = host.charAt(i);
+		    if (isstring = ! 
+			(((c <= '9') && (c >= '0')) || (c == '.'))) {
+			break;
+		    }
+		}
+		if ( isstring ) {
+		    // String host name, get all IP addresses:
+		    hostAddrs = InetAddress.getAllByName(host);
+		} else {
+		    // Numeric IP address, convert to InetAddress:
+		    hostAddrs    = new InetAddress[1];
+		    hostAddrs[0] = InetAddress.getByName(host);
+		}
+		setHostAddr(hostAddrs);
+	    } catch (UnknownHostException ex) {
+		String msg = ("The host name ["+host+"] couldn't be resolved. "
+			      + "Details: \""+ex.getMessage()+"\"");
+		throw new HttpException(ex, msg);
+	    }
+	}
+    }
+	    
     /**
      * HttpServer implementation - Get this servers' protocol.
      * @return A String encoding the protocol used to dialog with the target
@@ -147,6 +198,12 @@ public class HttpBasicServer extends HttpServer {
     {
 	// Create the new connection, try until all ip addrs have been checked
 	HttpBasicConnection conn = null;
+	try {
+	    updateHostAddr();
+	} catch (HttpException hex) {
+	    // unable to update, will continue with the old version
+	    // as it may be a temporary DNS problem
+	}
 	for (int loop = 0 ; loop < addrs.length ; loop++) {
 	    InetAddress addr = null;
 	    int         i    = addrptr;
@@ -250,21 +307,29 @@ public class HttpBasicServer extends HttpServer {
 		try {
 		    conn.input.available();
 		} catch (IOException ex) {
-		    // conn was closed
+		    // mark that connection as dead
+		    unregisterConnection(conn);
+		    deleteConnection(conn);
+		    conn.close();
 		    break;
 		}
-		if ( conn.markUsed() )
-		    return conn;
+		synchronized (this) {
+		    if ( conn.markUsed() )
+			return conn;
+		}
 	    }
 	    // Negotiate the creation of a new connection:
 	    if ( manager.negotiateConnection(this) ) {
 		conn = allocateConnection();
-		if ( conn.markUsed() ) {
-		    return conn;
-		} else {
-		    // Failed to establish a fresh connection !
-		    return null;
+		synchronized (this) {
+		    if ( conn.markUsed() ) {
+			return conn;
+		    } else {
+			// Failed to establish a fresh connection !
+			return null;
+		    }
 		}
+
 	    }
 	    // Wait for a connection to become available:
 	    try {
@@ -329,28 +394,8 @@ public class HttpBasicServer extends HttpServer {
 	this.port    = port;
 	this.timeout = timeout;
 	// Get this server IP addreses:
-	try {
-	    // Is the given string a valid IP address ?
-	    int     hostlen  = host.length();
-	    boolean isstring = true;
-	    for (int i = 0 ; i < hostlen ; i++) {
-		char c = host.charAt(i);
-		if (isstring = ! (((c <= '9') && (c >= '0')) || (c == '.')))
-		    break;
-	    }
-	    if ( isstring ) {
-		// String host name, get all IP addresses:
-		this.addrs = InetAddress.getAllByName(host);
-	    } else {
-		// Numeric IP address, convert to InetAddress:
-		this.addrs    = new InetAddress[1];
-		this.addrs[0] = InetAddress.getByName(host);
-	    }
-	} catch (UnknownHostException ex) {
-	    String msg = ("The host name ["+host+"] couldn't be resolved. "
-			  + "Details: \""+ex.getMessage()+"\"");
-	    throw new HttpException(ex, msg);
-	}
+	this.lookupLimit = null;
+	updateHostAddr();
     }
 
     // FIXME doc
@@ -452,7 +497,7 @@ public class HttpBasicServer extends HttpServer {
 		request.emit(os, Request.EMIT_BODY|Request.EMIT_FOOTERS);
 		os.flush();
 	    }
-	    reply = (Reply) p.parse();
+	    reply = (Reply) p.parse(manager.isLenient());
 	    // "eat" the 100 replies
 	    while ((reply.getStatus() / 100) == 1 ) {
 		reply = (Reply) p.parse();
