@@ -1,5 +1,5 @@
 // HttpManager.java
-// $Id: HttpManager.java,v 1.52.4.5 2000/01/21 10:16:03 ylafon Exp $
+// $Id: HttpManager.java,v 1.52.4.11 2000/07/04 08:54:52 ylafon Exp $
 // (c) COPYRIGHT MIT and INRIA, 1996.
 // Please first read the full copyright statement in file COPYRIGHT.html
 
@@ -128,6 +128,11 @@ public class HttpManager implements PropertyMonitoring {
      */
     public static final 
     String ACCEPT_ENCODING_P = "org.w3c.www.protocol.http.acceptEncoding";
+    /**
+     * Header properties - are we parsing answers in a lenient way?
+     */
+    public static final 
+    String LENIENT_P = "org.w3c.www.protocol.http.lenient";
     /**
      * Header properties - Should we use a proxy ?
      */
@@ -153,7 +158,7 @@ public class HttpManager implements PropertyMonitoring {
      * The default value for the <code>User-Agent</code> header.
      */
     public static final
-    String DEFAULT_USER_AGENT = "Jigsaw/2.0.4";
+    String DEFAULT_USER_AGENT = "Jigsaw/2.0.5";
 
     /**
      * This array keeps track of all the created managers.
@@ -190,6 +195,7 @@ public class HttpManager implements PropertyMonitoring {
     protected int timeout = 300000;
     protected int conn_count = 0;
     protected int conn_max = 5;
+    protected boolean lenient = true;
 
     /**
      * Update the proxy configuration to match current properties setting.
@@ -256,7 +262,10 @@ public class HttpManager implements PropertyMonitoring {
 	    if ( ival >= 0 )
 		tpl.setMinFresh(ival);
 	    return true;
-	} else if ( name.equals(ONLY_IF_CACHED_P) ) {
+	} else if ( name.equals(LENIENT_P) ) {
+	    lenient = props.getBoolean(ONLY_IF_CACHED_P, lenient);
+	    return true;
+	} if ( name.equals(ONLY_IF_CACHED_P) ) {
 	    tpl.setOnlyIfCached(props.getBoolean(ONLY_IF_CACHED_P, false));
 	    return true;
 	} else if ( name.equals(USER_AGENT_P) ) {
@@ -374,6 +383,8 @@ public class HttpManager implements PropertyMonitoring {
 	ival = props.getInteger(MIN_FRESH_P, -1);
 	if ( ival >= 0 )
 	    tpl.setMinFresh(ival);
+	// general, lenient
+	manager.lenient = props.getBoolean(LENIENT_P, true);
 	// General, User agent
 	tpl.setValue("user-agent"
 		     , props.getString(USER_AGENT_P
@@ -391,6 +402,8 @@ public class HttpManager implements PropertyMonitoring {
 	    tpl.setValue("accept-encoding", sval);
 	// Maximum number of allowed connections:
 	manager.conn_max = props.getInteger(CONN_MAX_P, 5);
+	// timeout value
+	manager.timeout = props.getInteger(TIMEOUT_P, manager.timeout);
 	// Register ourself as a property observer:
 	props.registerObserver(manager);
 	// Register that manager in our knwon managers:
@@ -489,7 +502,7 @@ public class HttpManager implements PropertyMonitoring {
 	connectionsLru.toHead(conn);
 	HttpServerState ss = conn.getServer().getState();
 	ss.registerConnection(conn);
-	notify();
+	notifyAll();
     }
 
     /**
@@ -517,7 +530,7 @@ public class HttpManager implements PropertyMonitoring {
 	    --conn_count;
 	    if (debug)
 		System.out.println("+++ delete conn_count: " + conn_count);
-	    notify();
+	    notifyAll();
 	}
     }
 
@@ -788,6 +801,22 @@ public class HttpManager implements PropertyMonitoring {
 	}
     }
 
+    /**
+     * Global settings - set the HTTP parsing lenient or not.
+     * @param lenient, true by default, false to detect wrong servers
+     */
+    public void setLenient(boolean lenient) {
+	this.lenient = lenient;
+    }
+
+    /**
+     * Is this manager parsing headers in a lenient way?
+     * @return A boolean.
+     */
+    public boolean isLenient() {
+	return lenient;
+    }
+
     /**
      * Global settings - Set an optional proxy to use.
      * Set the proxy to which all requests should be targeted. If the
